//==========================================================================
//
//      at91rm9200_eth_drv.c
//
//      ATMEL AT91RM9200 Ethernet Driver
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 2005 eCosCentric LTD
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):           ilko iliev
// Contributors:        ilko iliev
// Date:                04.07.2005
// Purpose:             AT91RM9200 ethernet driver
// Description:
//
//####DESCRIPTIONEND####
//
//==========================================================================

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/hal_arch.h>
#include <cyg/infra/diag.h>
#include <cyg/hal/drv_api.h>
#include <cyg/io/eth/netdev.h>
#include <cyg/io/eth/eth_drv.h>
#include <cyg/error/codes.h>
#include <pkgconf/devs_eth_arm_at91rm9200.h>
#include <cyg/hal/hal_platform_ints.h>
#include <cyg/hal/at91rm9200.h>

#include <cyg/hal/hal_io.h>

#include "phy.h"

#define NB_ETH_RX_PACKETS		128			// Number of ethernet packets buffer (max 1024)
#define ETH_PACKET_SIZE 		1536        // Size of an ethernet packet

#define	ETHER_ADDR_LEN			6


typedef struct
{
	cyg_uint32	sign;
	cyg_uint8  *rx_buff;

	cyg_uint8 esa[ETHER_ADDR_LEN];
  	
	int		tdlist_idx;

	// statistics
	//===========	
	int		cnt_bna;
	int		cnt_rec;
	int		cnt_ovr;
	int		cnt_abt;
	
	
    cyg_uint32 interrupts;
    cyg_uint32 rx_count;
    cyg_uint32 rx_deliver;
    cyg_uint32 tx_count;

/*
	int		cnt_tx_ok;
	int		cnt_scol;
	int		cnt_mcol;
	int		cnt_rx_ok;
	int		cnt_seqe;
	int		cnt_ale;
	int		cnt_dte;
	int		cnt_lcol;
	int		cnt_ecol;
	int		cnt_cse;
	int		cnt_tue;
	int		cnt_cde;
	int		cnt_elr;
	int		cnt_rjb;
	int		cnt_usf;
	int		cnt_sqee;
	int		cnt_drfc;
*/	

} EMAC_CHAN;


static EMAC_CHAN Emac_chan;
static cyg_uint8 TxBuffer[ETH_PACKET_SIZE];

typedef struct
{
	cyg_uint32	addr;	
	cyg_uint32	size;	
	
} TDLIST;


static TDLIST TdList[NB_ETH_RX_PACKETS] __attribute__((aligned(0x1000)));			// workaround for errata #11
static char  RxPacketBuff[NB_ETH_RX_PACKETS][ETH_PACKET_SIZE];


static void SetMAC(cyg_uint8 *esa);



ETH_DRV_SC( at91rm9200_sc,
           (void *)&Emac_chan,     		// driver specific data
           "eth0",                      // Name for this interface
           at91rm9200_start,
           at91rm9200_stop,
           at91rm9200_control,
           at91rm9200_can_send,
           at91rm9200_send,
           at91rm9200_recv,
           at91rm9200_deliver,
           at91rm9200_poll,
           at91rm9200_int_vector);

NETDEVTAB_ENTRY(at91rm9200_netdev,
                "ETH_DRV",
                at91rm9200_init,
                &at91rm9200_sc);
                
                
                



//==================================================================================
//							MiiStationWrite()
//							-----------------		
//	Parameter:
//		RegAddr   -> Register address. Specifies the register in the PHY to access.
//		PhyAddr   -> PHY address	
//		PhyWrData -> data to write to PHY register
//
//	Return: none
//==================================================================================
void MiiStationWrite(cyg_uint32 RegAddr, cyg_uint32 PhyAddr, cyg_uint32 PhyWrData)
{
unsigned int value = 0x50020000 | (RegAddr << 18) | (PhyAddr << 24);

	AT91C_BASE_EMAC->EMAC_MAN = value;
}

//==================================================================================
//							MiiStationRead()
//							----------------
//	Parameter:
//		RegAddr -> Register address. Specifies the register in the PHY to access.
//		PhyAddr -> PHY address	
//
//	Return:
//		PHY register 
//==================================================================================
cyg_uint32 MiiStationRead(cyg_uint32 RegAddr, cyg_uint32 PhyAddr)
{
unsigned int value = 0x60020000 | (RegAddr << 18) | (PhyAddr << 24);

	AT91C_BASE_EMAC->EMAC_CTL |= AT91C_EMAC_MPE;

	AT91C_BASE_EMAC->EMAC_MAN = value;

  	while(!((AT91C_BASE_EMAC->EMAC_SR) & AT91C_EMAC_IDLE));

	AT91C_BASE_EMAC->EMAC_CTL &= ~AT91C_EMAC_MPE;

	return (AT91C_BASE_EMAC->EMAC_MAN & 0x0000ffff);
  
 }



//==================================================================================
//								emac_isr()
//								----------
//
//==================================================================================
static cyg_uint32 emac_isr(cyg_vector_t vector, cyg_addrword_t data)
{
struct eth_drv_sc *sc = (struct eth_drv_sc *)data;
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);
volatile cyg_uint32 stat_isr, stat_rsr;


	// MAC Interrupt Status register indicates what interrupts are pending.
	//   It is automatically cleared once read. 
	//====================================================================
	stat_isr = AT91C_BASE_EMAC->EMAC_ISR;
	stat_rsr = AT91C_BASE_EMAC->EMAC_RSR;

	chan->interrupts++;

#ifdef AT91RM9200_ETH_STATS	

	if ( stat_rsr & AT91C_EMAC_RSR_BNA )
		chan->cnt_bna++;
  
	if ( stat_rsr & AT91C_EMAC_RSR_REC )
		chan->cnt_rec++;
  
	if ( stat_rsr & AT91C_EMAC_RSR_OVR )
		chan->cnt_ovr++;
  
  	if ( stat_isr & AT91C_EMAC_HRESP )
  		chan->cnt_abt++;
  
#endif  
  
  	// block this interrupt until the dsr completes
  	//=============================================
  	cyg_drv_interrupt_mask(vector);
  
  	// tell ecos to allow further interrupt processing
  	//================================================
  	cyg_drv_interrupt_acknowledge(vector);
  
  	return CYG_ISR_CALL_DSR;      // call the dsr
}

//==================================================================================
//								emac_dsr()
//								----------
//
//==================================================================================
static void emac_dsr(cyg_vector_t vector, cyg_ucount32 count, cyg_addrword_t data)
{
  eth_drv_dsr( vector, count, data );
  
  cyg_drv_interrupt_unmask(vector);
}


static void at91rm9200_pio_init(void)
{
	AT91C_BASE_PMC->PMC_PCER = 	((unsigned) 1 << AT91C_ID_EMAC);

	AT91C_BASE_PIOA->PIO_ASR = 
		((unsigned) AT91C_PA14_ERXER   ) |
		((unsigned) AT91C_PA12_ERX0    ) |
		((unsigned) AT91C_PA13_ERX1    ) |
		((unsigned) AT91C_PA8_ETXEN   ) |
		((unsigned) AT91C_PA16_EMDIO   ) |
		((unsigned) AT91C_PA9_ETX0    ) |
		((unsigned) AT91C_PA10_ETX1    ) |
		((unsigned) AT91C_PA11_ECRS_ECRSDV) |
		((unsigned) AT91C_PA15_EMDC    ) |
		((unsigned) AT91C_PA7_ETXCK_EREFCK);
	AT91C_BASE_PIOA->PIO_BSR = 0;
	AT91C_BASE_PIOA->PIO_PDR = 
		((unsigned) AT91C_PA14_ERXER   ) |
		((unsigned) AT91C_PA12_ERX0    ) |
		((unsigned) AT91C_PA13_ERX1    ) |
		((unsigned) AT91C_PA8_ETXEN   ) |
		((unsigned) AT91C_PA16_EMDIO   ) |
		((unsigned) AT91C_PA9_ETX0    ) |
		((unsigned) AT91C_PA10_ETX1    ) |
		((unsigned) AT91C_PA11_ECRS_ECRSDV) |
		((unsigned) AT91C_PA15_EMDC    ) |
		((unsigned) AT91C_PA7_ETXCK_EREFCK);

	AT91C_BASE_PIOB->PIO_ASR = 0;
	AT91C_BASE_PIOB->PIO_BSR =
		((unsigned) AT91C_PB12_ETX2) |
		((unsigned) AT91C_PB13_ETX3) |
		((unsigned) AT91C_PB14_ETXER) |
		((unsigned) AT91C_PB15_ERX2) |
		((unsigned) AT91C_PB16_ERX3) |
		((unsigned) AT91C_PB17_ERXDV) |
		((unsigned) AT91C_PB18_ECOL) |
		((unsigned) AT91C_PB19_ERXCK);
	AT91C_BASE_PIOB->PIO_PDR =
		((unsigned) AT91C_PB12_ETX2) |
		((unsigned) AT91C_PB13_ETX3) |
		((unsigned) AT91C_PB14_ETXER) |
		((unsigned) AT91C_PB15_ERX2) |
		((unsigned) AT91C_PB16_ERX3) |
		((unsigned) AT91C_PB17_ERXDV) |
		((unsigned) AT91C_PB18_ECOL) |
		((unsigned) AT91C_PB19_ERXCK);
}




//==================================================================================
//								at91rm9200_init()
//								-----------------
//
//==================================================================================
static bool at91rm9200_init(struct cyg_netdevtab_entry *tab)
{
struct eth_drv_sc *sc = (struct eth_drv_sc *)tab->device_instance;
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);
static cyg_interrupt emac_int_object;
static cyg_handle_t  emac_int_handle;
int ii, stat;
cyg_uint32 status;

	memset( chan, 0, sizeof(EMAC_CHAN));
	chan->sign = 123456;

	chan->esa[0] = 0x02;
	chan->esa[1] = 0x88;
	chan->esa[2] = 0x12;
	chan->esa[3] = 0x34;
	chan->esa[4] = 0x56;
	chan->esa[5] = 0x78;

	// Initialise tdList descriptor. This descriptor must be WORD aligned
	//===================================================================
	for (ii = 0; ii < NB_ETH_RX_PACKETS; ii++) 
	{
		TdList[ii].addr = (unsigned int) &RxPacketBuff[ii];
		TdList[ii].size = 0;
	}
	
	// Set the WRAP bit at the end of the list descriptor
	//===================================================
	TdList[NB_ETH_RX_PACKETS-1].addr |= 0x02;

	AT91C_BASE_EMAC->EMAC_CTL  = 0;

	at91rm9200_pio_init();
	
	AT91C_BASE_EMAC->EMAC_CTL |= AT91C_EMAC_MPE;				// enable MDIO interface
	
	stat = PhyReset();						
	if ( stat ) return false;

	status = PhyStatus();	

	AT91C_BASE_EMAC->EMAC_CFG = 0x800;							// reset state

	if ( status & PhyStatus_FullDuplex )
		AT91C_BASE_EMAC->EMAC_CFG |= AT91C_EMAC_FD;    			//set MII for 100BaseTX and Full Duplex


    if ( status & PhyStatus_100Mb )
		AT91C_BASE_EMAC->EMAC_CFG |=  AT91C_EMAC_SPD;    		//set MII for 10BaseT and Full Duplex 

		
	

  	// Ethernet Controller Initializatition
	//=====================================
	SetMAC( chan->esa );
	
	AT91C_BASE_EMAC->EMAC_RBQP = (unsigned int)TdList;
	AT91C_BASE_EMAC->EMAC_RSR  |= (AT91C_EMAC_RSR_OVR | AT91C_EMAC_RSR_REC | AT91C_EMAC_RSR_BNA);
	AT91C_BASE_EMAC->EMAC_CFG  |= (AT91C_EMAC_CAF) ; 
	AT91C_BASE_EMAC->EMAC_TAR = (cyg_uint32)TxBuffer;


	// enable interrupts
	//==================
	AT91C_BASE_EMAC->EMAC_IER = AT91C_EMAC_RCOM | AT91C_EMAC_RBNA
								| AT91C_EMAC_TUND | AT91C_EMAC_RTRY | AT91C_EMAC_TCOM
								| AT91C_EMAC_ROVR | AT91C_EMAC_HRESP;
	
  
  	cyg_drv_interrupt_create(AT91C_ID_EMAC,   				// Interrupt Vector
                           0,                           	// Interrupt Priority
                           (cyg_addrword_t)sc,  			// Reference to driver instance
                           emac_isr,
                           emac_dsr,
                           &emac_int_handle,
                           &emac_int_object);
  
  	cyg_drv_interrupt_attach(emac_int_handle);
  
  	sc->funs->eth_drv->init(sc, chan->esa);
  
  	return true;
}

//==================================================================================
//								at91rm9200_recv()
//								-----------------
//
//==================================================================================
static void at91rm9200_recv(struct eth_drv_sc *sc, struct eth_drv_sg *sg_list, int sg_len)
{
int i, len;
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);
cyg_uint8 *data, *buf = chan->rx_buff;

	chan->rx_count++;
  
	for (i = 0;  i < sg_len;  i++) 
	{
		data = (unsigned char *)(sg_list[i].buf);
		len = sg_list[i].len;
		if (len)
	  	{
	  		memcpy( data, buf, len );
	    	buf += len;
	  	}
	  	
	  	TdList[chan->tdlist_idx].addr &= ~0x01;
	}
	
#ifdef AT91RM9200_ETH_STATS	
	chan->cnt_tx_ok += AT91C_BASE_EMAC->EMAC_FRA   & ((1<<24)-1);  
	chan->cnt_scol  += AT91C_BASE_EMAC->EMAC_SCOL  & ((1<<16)-1);  
	chan->cnt_mcol  += AT91C_BASE_EMAC->EMAC_MCOL  & ((1<<16)-1);  
	chan->cnt_rx_ok += AT91C_BASE_EMAC->EMAC_OK    & ((1<<24)-1);  
	chan->cnt_seqe  += AT91C_BASE_EMAC->EMAC_SEQE  & ((1<<8)-1);   
	chan->cnt_ale   += AT91C_BASE_EMAC->EMAC_ALE   & ((1<<8)-1);     
	chan->cnt_dte   += AT91C_BASE_EMAC->EMAC_DTE   & ((1<<16)-1);    
	chan->cnt_lcol  += AT91C_BASE_EMAC->EMAC_LCOL  & ((1<<8)-1);    
	chan->cnt_ecol  += AT91C_BASE_EMAC->EMAC_ECOL  & ((1<<8)-1); 
	chan->cnt_cse   += AT91C_BASE_EMAC->EMAC_CSE   & ((1<<8)-1); 
	chan->cnt_tue   += AT91C_BASE_EMAC->EMAC_TUE   & ((1<<8)-1); 
	chan->cnt_cde   += AT91C_BASE_EMAC->EMAC_CDE   & ((1<<8)-1); 
	chan->cnt_elr   += AT91C_BASE_EMAC->EMAC_ELR   & ((1<<8)-1); 
	chan->cnt_rjb   += AT91C_BASE_EMAC->EMAC_RJB   & ((1<<8)-1); 
	chan->cnt_usf   += AT91C_BASE_EMAC->EMAC_USF   & ((1<<8)-1); 
	chan->cnt_sqee  += AT91C_BASE_EMAC->EMAC_SQEE  & ((1<<8)-1); 
	chan->cnt_drfc  += AT91C_BASE_EMAC->EMAC_DRFC  & ((1<<16)-1); 
#endif	
}

//==================================================================================
//								at91rm9200_deliver()
//								--------------------
//
//==================================================================================
static void at91rm9200_deliver(struct eth_drv_sc *sc)
{
int ii;
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);
	  
	chan->rx_deliver++;

	for (ii = 0; ii < NB_ETH_RX_PACKETS; ii++) 
	{				
		if ( TdList[ii].addr & 0x1) 
		{
			if ( AT91C_BASE_EMAC->EMAC_RSR & AT91C_EMAC_RSR_REC )
				AT91C_BASE_EMAC->EMAC_RSR |= AT91C_EMAC_RSR_REC;

			chan->tdlist_idx = ii;

			chan->rx_buff = (cyg_uint8 *)&RxPacketBuff[ii];	

			sc->funs->eth_drv->recv( sc, (TdList[ii].size & 0x7FF) );
		}
	}
  
}

//==================================================================================
//								at91rm9200_can_send()
//								--------------------
//
//==================================================================================
static int at91rm9200_can_send(struct eth_drv_sc *sc)
{
	if ( !(AT91C_BASE_EMAC->EMAC_TSR & AT91C_EMAC_BNQ) )
    	return ENOERR;
 
  	return 1;
}

//==================================================================================
//								at91rm9200_send()
//								-----------------
//
//==================================================================================
static void at91rm9200_send(struct eth_drv_sc *sc, struct eth_drv_sg *sg_list, int sg_len, int total_len, unsigned long key)
{
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);
int ii, len;
cyg_uint8 *data;
cyg_uint8 *buf = TxBuffer;

	chan->tx_count++;

  
  	// Put data into buffer
  	//=====================
  	for (ii = 0;  ii < sg_len;  ii++) 
  	{
    	data = (unsigned char *)sg_list[ii].buf;
    	len = sg_list[ii].len;

		memcpy( buf, data, len);
       	buf += len;
  	}
  	
	if (AT91C_BASE_EMAC->EMAC_TSR & AT91C_EMAC_BNQ) 
	{
  		AT91C_BASE_EMAC->EMAC_TSR |= AT91C_EMAC_COMP;		
		
		// Set address of the data in the Transmit Address register 
		//=========================================================
		AT91C_BASE_EMAC->EMAC_TAR = (AT91_REG)TxBuffer;
		
		// Set length of the packet in the Transmit Control register
		//==========================================================
		AT91C_BASE_EMAC->EMAC_TCR = total_len;
		
		sc->funs->eth_drv->tx_done(sc, key, 0);
	}
	else
		sc->funs->eth_drv->tx_done(sc, key, -EIO);

}

static void SetMAC( cyg_uint8 *esa)
{
	// the sequence write EMAC_SA1L and write EMAC_SA1H must be respected
	//===================================================================
	AT91C_BASE_EMAC->EMAC_SA1L = ((int)esa[2] << 24) | ((int)esa[3] << 16) | ((int)esa[4] << 8) | esa[5];
	AT91C_BASE_EMAC->EMAC_SA1H = ((int)esa[0] << 8)  | esa[1];
}


//==================================================================================
//								at91rm9200_start()
//								-----------------
//
//==================================================================================
static void at91rm9200_start(struct eth_drv_sc *sc, unsigned char *enaddr, int flags)
{
	SetMAC(enaddr);                       // set MAC address
  	HAL_REORDER_BARRIER();

	// Enable Receive and Transmit 
	//============================
	AT91C_BASE_EMAC->EMAC_CTL |= (AT91C_EMAC_RE | AT91C_EMAC_TE);

  	HAL_REORDER_BARRIER();
  	cyg_drv_interrupt_unmask(AT91C_ID_EMAC);
  
}

//==================================================================================
//								at91rm9200_stop()
//								-----------------
//
//==================================================================================
static void at91rm9200_stop(struct eth_drv_sc *sc)
{
	// Disable Receiver and Transmitter 
	//=================================
	AT91C_BASE_EMAC->EMAC_CTL &= ~(AT91C_EMAC_TE | AT91C_EMAC_RE);

  	HAL_REORDER_BARRIER();
  	cyg_drv_interrupt_mask(AT91C_ID_EMAC);
}

//==================================================================================
//								at91rm9200_control()
//								--------------------
//
//==================================================================================
static int at91rm9200_control(struct eth_drv_sc *sc, unsigned long key, void *data, int len)
{
EMAC_CHAN *chan = (EMAC_CHAN *)(sc->driver_private);

  	switch (key)
    {
#if defined(CYGPKG_NET)      
     	case ETH_DRV_GET_IF_STATS_UD:
     	case ETH_DRV_GET_IF_STATS:
        {
        	struct ether_drv_stats *pp = (struct ether_drv_stats*)data;

          	unsigned linkStatus;

          	// Read link status to be up to date
          	//==================================
          	linkStatus = PhyStatus();
          	if (linkStatus & PhyStatus_FullDuplex)
            	pp->duplex = 3;
          	else
            	pp->duplex = 2;

          	if (linkStatus & PhyStatus_LinkUp)
            	pp->operational = 3;
          	else
	            pp->operational = 2;

			if (linkStatus & PhyStatus_100Mb)
            	pp->speed = 100000000;
          	else
            	pp->speed = 10000000;

			
    		pp->tx_good 			 += AT91C_BASE_EMAC->EMAC_FRA   & ((1<<24)-1);
    		pp->rx_good 			 += AT91C_BASE_EMAC->EMAC_OK    & ((1<<24)-1);
    		pp->tx_single_collisions += AT91C_BASE_EMAC->EMAC_SCOL  & ((1<<16)-1);  
    		pp->tx_mult_collisions   += AT91C_BASE_EMAC->EMAC_MCOL  & ((1<<16)-1);
    		pp->tx_late_collisions   += AT91C_BASE_EMAC->EMAC_LCOL  & ((1<<8)-1);    
    		pp->tx_underrun			 += AT91C_BASE_EMAC->EMAC_TUE   & ((1<<8)-1);
    		pp->tx_carrier_loss		 += AT91C_BASE_EMAC->EMAC_CSE   & ((1<<8)-1);
    		pp->tx_deferred			 += AT91C_BASE_EMAC->EMAC_DTE   & ((1<<16)-1);
    		pp->rx_align_errors		 += AT91C_BASE_EMAC->EMAC_ALE   & ((1<<8)-1);
    		pp->tx_sqetesterrors	 += AT91C_BASE_EMAC->EMAC_SQEE  & ((1<<8)-1);
    		pp->rx_crc_errors		 += AT91C_BASE_EMAC->EMAC_SEQE  & ((1<<8)-1);
    		pp->rx_overrun_errors	 += AT91C_BASE_EMAC->EMAC_DRFC  & ((1<<16)-1); 
    		pp->rx_too_long_frames	 += AT91C_BASE_EMAC->EMAC_ELR   & ((1<<8)-1);
    		pp->rx_short_frames		 += AT91C_BASE_EMAC->EMAC_USF   & ((1<<8)-1);
    		pp->rx_resource_errors	 += AT91C_BASE_EMAC->EMAC_CDE   & ((1<<8)-1);
    		pp->tx_max_collisions    += AT91C_BASE_EMAC->EMAC_ECOL  & ((1<<8)-1);
    		
          	return ENOERR;
        }
#endif      

		// set new MAC address
		//====================
		case ETH_DRV_SET_MAC_ADDRESS: 
		{
		 	if (ETHER_ADDR_LEN != len)
		    	return -1;
		     
		 	at91rm9200_stop(sc);
		 
		 	memcpy( chan->esa, data, ETHER_ADDR_LEN );
		 
		 	at91rm9200_start(sc, data, 0);
		
		 	return ENOERR;
		}
	     
	     
#ifdef	ETH_DRV_GET_MAC_ADDRESS
		// get current MAC address
		//========================
	    case ETH_DRV_GET_MAC_ADDRESS: 
	    {
	         if (len < ETHER_ADDR_LEN)
	             return -1;
	             
	         memcpy(data, chan->esa, ETHER_ADDR_LEN);
	         return ENOERR;
	    }
#endif
	
    	case ETH_DRV_SET_MC_LIST:
    	case ETH_DRV_SET_MC_ALL:
      		return ENOERR;
	

     	default:
      		return -1;
    }


}

//==================================================================================
//								at91rm9200_poll()
//								-----------------
//
//==================================================================================
static void at91rm9200_poll(struct eth_drv_sc *sc)
{
}

//==================================================================================
//								at91rm9200_int_vector()
//								-----------------------
//
//==================================================================================
static int at91rm9200_int_vector(struct eth_drv_sc *sc)
{
  return AT91C_ID_EMAC;
}


