//==========================================================================
//
//      eb9200_nand_flash.c
//
//      Flash programming for NAND FLASH device on EB9200 board
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2003 Nick Garnett <nickg@calivar.com>
// Copyright (C) 2003, 2004 Gary Thomas
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
//
// Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
// at http://sources.redhat.com/ecos/ecos-license/
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    ilko iliev
// Contributors: 
// Date:         2006-04-24
// Purpose:      
// Description:  
//              
//####DESCRIPTIONEND####
//
//==========================================================================

#include <cyg/infra/cyg_type.h>
#include <cyg/hal/hal_io.h>             // I/O macros
#include <pkgconf/hal.h>
#include <pkgconf/system.h>
#include <cyg/io/user_config_keys.h>
#include <cyg/infra/diag.h>
#include <cyg/io/devtab.h>
#include <pkgconf/devs_flash_nand_arm_eb9200.h>


static bool nandio_init(struct cyg_devtab_entry *tab);
static Cyg_ErrNo nandio_lookup(struct cyg_devtab_entry **tab, struct cyg_devtab_entry *sub_tab, const char *name);
static Cyg_ErrNo nandio_get_config(cyg_io_handle_t handle,  cyg_uint32 key, void *buf, cyg_uint32 *len);
static Cyg_ErrNo nandio_set_config(cyg_io_handle_t handle, cyg_uint32 key, const void *buf, cyg_uint32* len);
static Cyg_ErrNo nandio_bwrite(cyg_io_handle_t handle, const void *buf, cyg_uint32 *len, cyg_uint32 pos);
static Cyg_ErrNo nandio_bread(cyg_io_handle_t handle, void *buf, cyg_uint32 *len, cyg_uint32 pos);


/*
#define NAND_FLASH_BASE				(0x40000000)
#define NAND_FLASH_CLE				(CYGNUM_FLASH_BASE + (1 << 21) )
#define NAND_FLASH_ALE				(CYGNUM_FLASH_BASE + (1 << 22) )
*/

typedef struct
{
	int	bus_width;			// 8-bit or 16-bit NAND Flash chip
	
	cyg_uint32 io_data;
	cyg_uint32 io_cmd;
	cyg_uint32 io_addr;
	
} NANDIO_CHAN;


BLOCK_DEVIO_TABLE( nandio_handlers,
                   nandio_bwrite,			// bwrite
                   nandio_bread,			// bread
                   NULL,                    // select is not used
                   nandio_get_config,		// get_config	
                   nandio_set_config		// set_config
                 );

#ifdef CYGPKG_DEVS_FLASH_NANDIO_0
	static NANDIO_CHAN Nandio_chan0 = 
	{
		.bus_width     = CYGDAT_DEVS_FLASH_NANDIO_BUSWIDTH_0,
		.io_data = CYGDAT_DEVS_FLASH_NANDIO_DATABASE0,
		.io_cmd  = CYGDAT_DEVS_FLASH_NANDIO_CMDBASE0,	
		.io_addr = CYGDAT_DEVS_FLASH_NANDIO_ADDRBASE0,
	};
	
	BLOCK_DEVTAB_ENTRY(eb9200_nandio_0,
	                  CYGDAT_DEVS_FLASH_NANDIO_NAME_0,
	                  NULL,                                             // Base device name
	                  &nandio_handlers,
	                  nandio_init,
	                  nandio_lookup,
	                  &Nandio_chan0);                          			// Private data pointer
#endif	                  


#ifdef CYGPKG_DEVS_FLASH_NANDIO_1
	static NANDIO_CHAN Nandio_chan1 = 
	{
		.bus_width     = CYGDAT_DEVS_FLASH_NANDIO_BUSWIDTH_1,
		.io_data = CYGDAT_DEVS_FLASH_NANDIO_DATABASE1,
		.io_cmd  = CYGDAT_DEVS_FLASH_NANDIO_CMDBASE1,	
		.io_addr = CYGDAT_DEVS_FLASH_NANDIO_ADDRBASE1,
	};

	BLOCK_DEVTAB_ENTRY(eb9200_nandio_1,
	                  CYGDAT_DEVS_FLASH_NANDIO_NAME_1,
	                  NULL,                                             // Base device name
	                  &nandio_handlers,
	                  nandio_init,
	                  nandio_lookup,
	                  &Nandio_chan1);                          			// Private data pointer
#endif


static bool nandio_init(struct cyg_devtab_entry *tab)
{
NANDIO_CHAN *chan = (NANDIO_CHAN *)tab->priv;

	chan->bus_width = 8;

	return ENOERR;	
}




static Cyg_ErrNo nandio_lookup(struct cyg_devtab_entry **tab, struct cyg_devtab_entry *sub_tab, const char *name)
{
NANDIO_CHAN *chan = (NANDIO_CHAN *)(*tab)->priv;
	
	AT91F_NAND_CfgPIO( chan->bus_width );

#if 0
// test area begin	
volatile unsigned char *pp;
int ii=1;

switch(ii)
{
	case 0:
		pp = (unsigned char *)chan->io_data;
		break;

	case 1:
		pp = (unsigned char *)chan->io_addr;
		break;
		
	case 2:
		pp = (unsigned char *)chan->io_cmd;
		break;
		
}

while(1)
{
	AT91F_NAND_assert_CS();
	
	for (ii=0; ii < 8; ii++ )
		*pp = (1 << ii);


	AT91F_NAND_release_CS();
	AT91F_NAND_release_CS();
	AT91F_NAND_release_CS();
	AT91F_NAND_release_CS();
	AT91F_NAND_release_CS();
}
// test area end	
#endif
	
	return ENOERR;	
}

//=============================================================================
//							nandio_bread()
//							--------------
//	Parameter:
//		handle ->
//		buf    ->
//		len    ->
//		pos    ->
//
//	Return:
//		ENOERR
//		EINVAL 
//
//	Description:
//		Read data from the NAND Flash device
//=============================================================================
static Cyg_ErrNo nandio_bread(cyg_io_handle_t handle, void *buf, cyg_uint32 *len, cyg_uint32 key )
{
cyg_devtab_entry_t *tab = (cyg_devtab_entry_t *)handle;
NANDIO_CHAN *chan = (NANDIO_CHAN *)tab->priv;
int ii;

	if ( chan->bus_width == 8 )
	{
		volatile cyg_uint8 *pp = (cyg_uint8 *)chan->io_data;
		volatile cyg_uint32 *fpp = (cyg_uint32 *)chan->io_data;
		cyg_uint8 *dest = buf;
		cyg_uint32 *fdest = buf;

		if ( ((*len) % 4 ) == 0 )
		{
			for ( ii=0; ii < (*len)/4; ii++)
				*fdest++ = *fpp;			
		}
		else
		{
			for ( ii=0; ii < *len; ii++)
				*dest++ = *pp;			
		}
	}
	else
	if ( chan->bus_width == 16 )
	{
		if ( key == CYG_IO_CONFIG_FLASH_NAND_DATA )
		{
			volatile cyg_uint16 *pp = (cyg_uint16 *)chan->io_data;
			cyg_uint16 *dest = buf;
		
			for ( ii=0; ii < *len; ii++)
				*dest++ = *pp;			
		}		
		else
		{
			// read the ID
			//============
			volatile cyg_uint16 *pp = (cyg_uint16 *)chan->io_data;
			cyg_uint8 *dest = buf;
			
			for ( ii=0; ii < (*len); ii++)
				*dest++ = (cyg_uint8)(*pp);			
		}
				
	}
	else
		return EINVAL;	
	
	return ENOERR;	
}



static Cyg_ErrNo nandio_bwrite(cyg_io_handle_t handle, const void *buf, cyg_uint32 *len, cyg_uint32 key )
{
cyg_devtab_entry_t *tab = (cyg_devtab_entry_t *)handle;
NANDIO_CHAN *chan = (NANDIO_CHAN *)tab->priv;
volatile void *dest;
int ii;

	switch( key )
	{
		// write array of data
		//====================
		case CYG_IO_CONFIG_FLASH_NAND_DATA:
			dest = (void *)chan->io_data;
			break;

		// write array of commands
		//========================	
		case CYG_IO_CONFIG_FLASH_NAND_CMD:
			dest = (void *)chan->io_cmd;
			break;

		// write array of addresses
		//=========================
		case CYG_IO_CONFIG_FLASH_NAND_ADDR:
			dest = (void *)chan->io_addr;
			break;
			
		default:
			return EINVAL;			
	}


	if ( chan->bus_width == 8 )
	{
		cyg_uint8 *src = (cyg_uint8 *)buf;
		
		for ( ii=0; ii < *len; ii++)
			*(cyg_uint8 *)dest = *src++;			
	}
	else
	if ( chan->bus_width == 16 )
	{
		if ( key == CYG_IO_CONFIG_FLASH_NAND_DATA )
		{
			// data write
			//===========
			cyg_uint16 *src = (cyg_uint16 *)buf;
			
			for ( ii=0; ii < (*len)/2; ii++)
				*(cyg_uint16 *)dest = *src++;			
		}
		else
		{
			// command and address write
			//==========================
			cyg_uint8 *src = (cyg_uint8 *)buf;
			
			for ( ii=0; ii < (*len); ii++)
				*(cyg_uint16 *)dest = (cyg_uint16)(*src++);			
		}
	}
	else
		return EINVAL;	

	return ENOERR;
}


static Cyg_ErrNo nandio_get_config(cyg_io_handle_t handle,  cyg_uint32 key, void *buf, cyg_uint32 *len)
{
	switch( key )
	{
		case CYG_IO_CONFIG_FLASH_NAND_GET_RB_FUNC:
			if ( *len != 4 ) return EINVAL;
			*(int*)buf = (int)AT91F_NAND_is_BUSY_asserted;
			break;
		
		default:
			return ENOSUPP;	
	}
	
	return ENOERR;	
}

static Cyg_ErrNo nandio_set_config(cyg_io_handle_t handle, cyg_uint32 key, const void *buf, cyg_uint32* len)
{
cyg_devtab_entry_t *tab = (cyg_devtab_entry_t *)handle;
NANDIO_CHAN *chan = (NANDIO_CHAN *)tab->priv;

	switch( key )
	{
		// assert CS
		//==========	
		case CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS:
			AT91F_NAND_assert_CS();
			break;	
		
		// release CS
		//===========
		case CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS:
			AT91F_NAND_release_CS();
			break;	
			
		case CYG_IO_CONFIG_FLASH_NAND_SET_WIDTH:
			if ( *len != 4 ) return EINVAL;
			
			chan->bus_width = *(int *)buf;
			AT91F_NAND_CfgPIO( chan->bus_width );
			break;
	}
	
	return ENOERR;
}
