//===============================================================================
//                          class NAND
//                          ----------
//  Generic NAND Flash support
//
//	www.ronetix.at
//===============================================================================
#include <string.h>
#include <stdio.h>
#include <cyg/io/io.h>
#include <cyg/io/devtab.h>
#include <cyg/io/user_config_keys.h>
#include "nandio.h"

#define DEBUG_NAND

static const NAND_INFO NandInfo[] =
{
	// 	id 		page	spare	npages	nblocks		naddr	width
	//===============================================================================
	{  	0x6B, 	512, 	16, 	16, 	512, 		3, 		8	},		// 32 MBit
	{  	0xE3, 	512, 	16, 	16, 	512, 		3, 		8	},		// 32 MBit
	{  	0xE5, 	512, 	16, 	16, 	512, 		3, 		8	},		// 32 MBit

	{  	0xE6, 	512, 	16, 	16, 	1024, 		3, 		8	},		// 64 MB

	{  	0x73, 	512, 	16, 	32, 	1024, 		3, 		8	},		// 128 MB

	{  	0x35, 	512, 	16, 	32, 	2048, 		3, 		8	},		// 256 MB, 8-bit,  1.8V
	{  	0x75, 	512, 	16, 	32, 	2048, 		3, 		8	},		// 256 MB, 8-bit,  3.3V
	{  	0x45, 	512, 	16, 	32, 	2048, 		3, 		16	},		// 256 MB, 16-bit, 1.8V
	{  	0x55, 	512, 	16, 	32, 	2048, 		3, 		16	},		// 256 MB, 16-bit, 3.3V

	{  	0x36, 	512, 	16, 	32, 	4096, 		4, 		8	},		// 512 MB, 8-bit,  1.8V	
	{  	0x76, 	512, 	16, 	32, 	4096, 		4, 		8	},		// 512 MB, 8-bit,  3.3V
	{  	0x46, 	512, 	16, 	32, 	4096, 		4, 		16	},		// 512 MB, 16-bit, 1.8V	
	{  	0x56, 	512, 	16, 	32, 	4096, 		4, 		16	},		// 512 MB, 16-bit, 3.3V

	{  	0x39, 	512, 	16, 	32, 	8192, 		4, 		8	},		// 1 GB, 8-bit,  1.8V	
	{  	0x78, 	512, 	16, 	32, 	8192, 		4, 		8	},		// 1 GB, 8-bit
	{  	0x79, 	512, 	16, 	32, 	8192, 		4, 		8	},		// 1 GB, 8-bit,  3.3V
	{  	0x49, 	512, 	16, 	32, 	8192, 		4, 		16	},		// 1 GB, 16-bit, 1.8V	
	{  	0x59, 	512, 	16, 	32, 	8192, 		4, 		16	},		// 1 GB, 16-bit, 3.3V

	{  	0xA1, 	2048, 	64, 	64, 	1024, 		4, 		8	},		// 1 GB, 8-bit,  1.8V
	{  	0xF1, 	2048, 	64, 	64, 	1024, 		4, 		8	},		// 1 GB, 8-bit,  3.3V
	{  	0xB1, 	2048, 	64, 	64, 	1024, 		4, 		16	},		// 1 GB, 16-bit, 1.8V
	{  	0xC1, 	2048, 	64, 	64, 	1024, 		4, 		16	},		// 1 GB, 16-bit, 3.3V

	{  	0xAA, 	2048, 	64, 	64, 	2048, 		5, 		8	},		// 2 GB, 8-bit,  1.8V
	{  	0xDA, 	2048, 	64, 	64, 	2048, 		5, 		8	},		// 2 GB, 8-bit,  3.3V
	{  	0xBA, 	2048, 	64, 	64, 	2048, 		5, 		16	},		// 2 GB, 16-bit, 1.8V
	{  	0xCA, 	2048, 	64, 	64, 	2048, 		5, 		16	},		// 2 GB, 16-bit, 3.3V

	{  	0xDC, 	2048, 	64, 	64, 	4096, 		5, 		8	},		// 4 GB, 8-bit
	{  	0xCC, 	2048, 	64, 	64, 	4096, 		5, 		16	},		// 4 GB, 16-bit
	
	{  	0xD3, 	2048, 	64, 	64, 	8192, 		5, 		8	},		// 8 GB, 8-bit


	// must be the last record
	//========================
	{  	0, 	0, 	0, 	0, 	0, 		0, 		0	}
};
	

externC void hal_delay_us(cyg_int32 usecs);



//====================================================================
//              NAND::init()
//              ------------
// Parameter:
//      none
//
//  Return:
//      ENOERR     - if no error
//
//	Description:
//		Read the NAND ID and look for bad blocks
//====================================================================
int NAND::init()
{
cyg_uint8 id[4];
cyg_uint32 value, len;

	reset();
    
    read_id( id );

	info = &NandInfo[0];
	
	while( info->id != 0 && info->id != id[1] ) info++;
	
	if ( info->id == 0 )
		return EIO;

	// set bus width: 8 or 16-bit
	//===========================
	value = info->width;
	len = 4;
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_SET_WIDTH, &value, &len );  

	// get pointer to function which return true is Read/Busy pin is '0'
	//==================================================================
	is_busy_asserted = NULL;
	len = 4;
	cyg_io_get_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_GET_RB_FUNC, &is_busy_asserted, &len );  


	// prepare the function pointers
	//===============================
	if ( IS_LARGE_PAGE(info->page_size) ) 
	{
		p_read_page = &NAND::page_read_large;
	}
	else
	{
		p_read_page = &NAND::page_read_small;
	}

	bad_blocks = new char[info->num_blocks];
	memset( bad_blocks, 0, info->num_blocks );
	num_bad_blocks = 0;	
	
	look_bad_blocks();
	
	print_info();
	
    return ENOERR;
}

//==================================================================================
//							ckeck_spare_bad()
//							-----------------
//
//	Return true if the spare is marked as bad
//==================================================================================
static int ckeck_spare_bad( cyg_uint32 *buff, int ndwords )
{
#define	BAD_BLOCK_PATTERN		0x01000100
int ii, mm=0, nn=0;

	for( ii=0; ii < ndwords; ii++ )
	{
		switch( buff[ii] )
		{
			case 0: 
				mm++; 
				break;	
			
			case BAD_BLOCK_PATTERN:
				nn++;
				break;
		}
	}

	return ( mm == ndwords || nn == ndwords ) ? true : false;
}


//==================================================================================
//						NAND::look_bad_blocks()
//						-----------------------
//	Look for bad blocks.
//  A block is bad if the spare info is filled with 0x00000000 or 0x01000100
//==================================================================================
int NAND::look_bad_blocks(void)
{
int ii, page_num, stat, spare_size = info->spare_size/4;
cyg_uint32 *spare = new cyg_uint32[spare_size];

	num_bad_blocks = 0;
	page_num = info->pages_per_block;	// begin from block[1]
	
	for (ii=1; ii < info->num_blocks; ii++ )
	{
		stat = page_read( page_num, NULL, (cyg_uint8 *)spare );
		if ( stat ) return stat;

		if ( ckeck_spare_bad( spare, spare_size ) )
			bad_blocks[ii] = true;
		else	
		{
			stat = page_read( page_num+1, NULL, (cyg_uint8 *)spare );
			if ( stat ) return stat;
			
			if ( ckeck_spare_bad( spare, spare_size ) )
				bad_blocks[ii] = true;
		}
		
		if ( bad_blocks[ii] == true )
			num_bad_blocks++;

		page_num += info->pages_per_block;
	}
	
	delete spare;

	return ENOERR;	
}

//==================================================================================
//						NAND::print_info()
//						------------------
//	Print some NAND Flash information
//==================================================================================
void NAND::print_info(void)
{
#ifdef DEBUG_NAND		
int ii, chip_size;

	chip_size = info->page_size * info->pages_per_block;
	chip_size /= 1024;		// convert to KB to prevent overflow
	chip_size *= info->num_blocks;
	
	printf("Nand Flash: ID = 0x%X, %d-bit, %d MB\r\n", info->id, info->width, chip_size/1024 );
	printf("  page size            = %d + %d bytes\r\n", info->page_size, info->spare_size );
	printf("  pages per block      = %d\r\n", info->pages_per_block );
	printf("  number of blocks     = %d\r\n", info->num_blocks );
	printf("  number of bad blocks = %d\r\n", num_bad_blocks );
	printf("  bad block list: ");	
	for ( ii=0; ii < info->num_blocks; ii++ )
		if ( bad_blocks[ii] == true )
			printf(" %d", ii );

	printf("\r\n\n");
#endif
}


//==================================================================================
//						NAND::mark_bad_block()
//						----------------------
//	Mark a block as bad
//==================================================================================
int NAND::mark_bad_block( int block_num )
{
cyg_uint8 *spare = new cyg_uint8[info->spare_size];
int stat, page_num;	
	
	memset( spare, 0, info->spare_size );		
	
	page_num = block_num * info->pages_per_block;
	stat = page_write( page_num, NULL, spare);
	if ( stat ) return stat;
	
	bad_blocks[block_num] = true;
	
	return ENOERR;	
}



//==================================================================================
//						NAND::is_bad()
//						--------------
//	Check if the page_num is in a bad block
//==================================================================================
int NAND::is_bad( int page_num )
{
int block_num = page_num / info->pages_per_block;

	return bad_blocks[block_num];
}


//==================================================================================
//						NAND::page_read()
//						-----------------
//	Read a small or large page
//==================================================================================
int NAND::page_read(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare)
{
	return (this->*p_read_page)( page_num, buff_data, buff_spare );
}


//==================================================================================
//                      NAND::page_read_small()
//                      -----------------------
//  Parameter:
//      page_num   - page number to read.
//		buff_data  - pointer to buffer where to store the data
//		buff_spare - pointer to buffer where to store the spare
//
//  Return:
//      ENOERR - if no error
//
//  Description:
//      Read one SMALL page ( 512 bytes) and/or spare information.
//==================================================================================
int NAND::page_read_small(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare )
{
cyg_uint8 cmd;
cyg_uint32 len;
	
	// assert CS
	//==========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, NULL, NULL );  

	if ( buff_data )
	{
		cmd = SM_CMD_READ_MODE1;					// Read data
		
		len = 1;
		cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );
	
	    write_addr( page_num, 0 );

	    if ( wait_busy(SM_TMOUT_READ) )
	    {
	        reset();
	        return EIO;
	    }

		len = info->page_size;		
		cyg_io_bread( nandio_handle, buff_data, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}
	

	if ( buff_spare )
	{
		if ( buff_data == NULL )
		{
			cmd = SM_CMD_READ_SPARE;					
			
			len = 1;
			cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );
		
		    write_addr( page_num, 0 );
	
		    if ( wait_busy(SM_TMOUT_READ) )
		    {
		        reset();
		        return EIO;
		    }
		}
		
		len = info->spare_size;		
		cyg_io_bread( nandio_handle, buff_spare, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}

	// release CS
	//===========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, NULL, NULL );  
	
    return ENOERR;
}

//==================================================================================
//                      NAND::page_read_large()
//                      -----------------------
//  Parameter:
//      page_num - page number to read.
//		buff_data  - pointer to buffer where to store the data
//		buff_spare - pointer to buffer where to store the spare
//
//  Return:
//      ENOERR - if no error
//
//  Description:
//      Read one LARGE page ( 2048 bytes) and/or spare information.
//==================================================================================
int NAND::page_read_large(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare )
{
cyg_uint8 cmd;
cyg_uint32 len;
	
	// assert CS
	//==========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, NULL, NULL );  

	
	if ( buff_data )
	{
		cmd = NAND_CMD_PAGE_READ1;
		len = 1;
		cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

		write_addr( page_num, 0 );
		
		cmd = NAND_CMD_PAGE_READ2;
		len = 1;
		cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

	    if ( wait_busy(SM_TMOUT_READ) )
	    {
	        reset();
	        return EIO;
	    }
		
		// re-issue the PAGE READ command
		//===============================
		cmd = NAND_CMD_PAGE_READ1;
		len = 1;
		cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );
	
		len = info->page_size;
		cyg_io_bread( nandio_handle, buff_data, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}
		
	if ( buff_spare )
	{

		if ( buff_data == NULL )
		{
			cmd = NAND_CMD_PAGE_READ1;
			len = 1;
			cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

			write_addr( page_num, info->page_size );
			
			cmd = NAND_CMD_PAGE_READ2;
			len = 1;
			cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );
	
		    if ( wait_busy(SM_TMOUT_READ) )
		    {
		        reset();
		        return EIO;
		    }
			
			// re-issue the PAGE READ command
			//===============================
			cmd = NAND_CMD_PAGE_READ1;
			len = 1;
			cyg_io_bwrite( nandio_handle, &cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );
		}
			
		len = info->spare_size;		
		cyg_io_bread( nandio_handle, buff_spare, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}
		
	// release CS
	//===========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, NULL, NULL );  
	
    return ENOERR;
}




//==================================================================================
//                      NAND::page_write()
//                      ------------------
//  Parameter:
//      page_num   - page number to write
//		buff_data  - pointer to buffer with data
//		buff_spare - pointer to buffer with spare
//
//  Return:
//      ENOERR - if no error
//
//  Description:
//      Write one page ( 512 or 2048 bytes ) and/or spare information.
//==================================================================================
int NAND::page_write(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare)
{
cyg_uint8 cmd[4];
cyg_uint32 len;
int offset;
	
	// check for bad block
	//====================
	if ( is_bad(page_num) )
		return EIO;

	// assert CS
	//==========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, NULL, NULL );  

	len = 0;
	offset = 0;
	if ( IS_LARGE_PAGE(info->page_size) == false )
	{
		// 512 byte small page
		//====================
		if( buff_data )
	    	cmd[0] = SM_CMD_READ_MODE1;					
		else	    	
		if( buff_spare )
	    	cmd[0] = SM_CMD_READ_SPARE;					
		else
			return EIO;	
			
		len = 1;	
	}
	else
	{
		// 2048 bytes large page
		//======================
		if( buff_spare && !buff_data )
	    	offset = info->page_size;							// write only the spare information
	}
	

	cmd[len++] = NAND_CMD_SERIAL_DATA_INPUT;
	
	cyg_io_bwrite( nandio_handle, cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

	write_addr( page_num, offset );

	if ( buff_data )
	{
		len = info->page_size;
		cyg_io_bwrite( nandio_handle, buff_data, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}
	
	if ( buff_spare )
	{
		len = info->spare_size;
		cyg_io_bwrite( nandio_handle, buff_spare, &len, CYG_IO_CONFIG_FLASH_NAND_DATA );
	}
	

    // Program the page and spare info
    //================================
    cmd[0] = NAND_CMD_AUTO_PROGRAM;
    len = 1;
	cyg_io_bwrite( nandio_handle, cmd, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

    if ( wait_busy(SM_TMOUT_READ) )
    {
        reset();
        return EIO;
    }
    
	// release CS
	//===========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, NULL, NULL );  
	
    return(ENOERR);
}



//=============================================================================
//						NAND::block_erase()
//						-------------------
//	Parameter:
//		num_block -> block  number to erase. If -1 then erase all blocks
//
//	Erase one block
//=============================================================================
int NAND::block_erase( int block_num )
{
int ii, stat;
cyg_uint8 buff[4];
cyg_uint32 len;
	
	if ( block_num == -1 )
	{
		for ( ii=1; ii < info->num_blocks; ii++)
		{	
			if ( bad_blocks[ii] )
				continue;
				
			stat = block_erase( ii );
			if ( stat ) 
				return stat;
		}
		
		return ENOERR;
	}
	
	if ( bad_blocks[block_num] )
		return EIO;
	
	if ( block_num >= info->num_blocks )
		return EINVAL;
	
	// assert CS
	//==========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, NULL, NULL );  

    // set block for erase
    //====================
    buff[0] = NAND_CMD_AUTO_BLOCK_ERASE1;
    len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

	write_addr_erase( block_num );
		
	buff[0] = NAND_CMD_AUTO_BLOCK_ERASE2;
	len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

    if ( wait_busy(SM_TMOUT_ERASE) )
    {
        reset();
        return EIO;
    }

	// release CS
	//===========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, NULL, NULL );  

    return(ENOERR);
}


//=============================================================================
//						NAND::write_addr()
//						------------------
//	Parameter:
//		page_num -> page number
//		offset   -> offset in the page
//
//	Return:
//		none
//
//	Description:
//		Write the 3,4 or 5 cycle address to the nand flash
//=============================================================================
void NAND::write_addr( cyg_uint32 page_num, int offset )
{
cyg_uint8 buff[8];
cyg_uint32 len;

	if ( IS_LARGE_PAGE(info->page_size) == false )
	{
		switch( info->n_addr_cycles )
		{
			case 3:
	    		buff[0] = 0;
	    		buff[1] = (cyg_uint8)page_num;
	    		buff[2] = (cyg_uint8)(page_num/0x0100);
	    		len = 3;
				break;
				
			case 4:
	    		buff[0] = 0;
	    		buff[1] = (cyg_uint8)page_num;
	    		buff[2] = (cyg_uint8)(page_num/0x0100);
	    		buff[3] = (cyg_uint8)(page_num/0x010000);
	    		len = 4;
				break;
				
			case 5:
	    		buff[0] = 0;
	    		buff[1] = 0;
	    		buff[2] = (cyg_uint8)page_num;
	    		buff[3] = (cyg_uint8)(page_num/0x0100);
	    		buff[4] = (cyg_uint8)(page_num/0x010000);
	    		len = 5;
				break;		
		}
	}
	else
	{
		if ( info->width == 16 )
			offset >>= 1;
		
		len = 4;
		buff[0] = (cyg_uint8)(offset & 0xFF);
		buff[1] = (cyg_uint8)((offset >> 8) & 0xFF);
		
		buff[2] = (cyg_uint8)(page_num & 0xFF);
		buff[3] = (cyg_uint8)((page_num >> 8) & 0xFF);
		
		if ( info->n_addr_cycles == 5 )
		{
			buff[4] = (cyg_uint8)((page_num >> 16) & 0xFF);
			len++;
		}			

	}
		
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_ADDR );
}


//=============================================================================
//						NAND::write_addr_erase()
//						------------------------
//	Parameter:
//		block_num -> block_number 
//
//	Return: none
//
//	Description:
//		Write the address for the "erase" command
//=============================================================================
void NAND::write_addr_erase(int block_num)
{
cyg_uint8 buff[8];
cyg_uint32 len, page_num;

	page_num = block_num * info->pages_per_block;

	if ( IS_LARGE_PAGE(info->page_size) == false )
	{
		buff[0] = (cyg_uint8)page_num;
		buff[1] = (cyg_uint8)page_num/256;

		if ( info->n_addr_cycles > 3 )
		{
			buff[2] = (cyg_uint8)(page_num/65536);
			len = 3;
		}
		else
			len = 2;
	}
	else
	{
		buff[0] = (cyg_uint8)page_num;
		buff[1] = (cyg_uint8)(page_num >> 8);
		buff[2] = (cyg_uint8)(page_num >> 16);
		len = 3;
	}
		
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_ADDR );	
	
}



//=====================================================
//              NAND::wait_busy()
//              -----------------
//
// Parameter:
//  timeout - timeout in millisec
//
//=====================================================
int NAND::wait_busy(int timeout)
{
cyg_tick_count_t tm1, tm2;
char buff[4];
cyg_uint32 len;

	if ( is_busy_asserted )
	{
		// use the Ready/Busy pin
		//=======================
		tm1 = cyg_current_time() + timeout;
		while ( is_busy_asserted() && cyg_current_time() < tm1 );
		
		if ( is_busy_asserted() ) return EBUSY;
		
		return ENOERR;	
	}
	

	// send STATUS_READ command
	//=========================
	buff[0] = SM_CMD_STATUS_READ;
	len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

	tm1 = cyg_current_time() + timeout;
	do
	{
		buff[0] = 0;
		len = 1;
		cyg_io_bread( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_READ8 );
		
		if ( buff[0] & (1 << 5) )
			break;
		
		tm2 = cyg_current_time();
	}
	while( 	tm2 < tm1 );
	
	if ( buff[0] & (1 << 5) == 0 )
		return EIO;	


    return ENOERR;
}


//=====================================================
//              NAND::reset()
//              -------------
//
//  	Reset the NAND Flash
//=====================================================
void NAND::reset(void)
{
char buff[4];
cyg_uint32 len;

	// release CS
	//===========
	len   = 4;
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, buff, &len );  

	// assert CS
	//==========
	len   = 4;
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, buff, &len );  
	
	buff[0] = SM_CMD_RESET;
	buff[1] = 0;
	len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

//    wait_busy( SM_TMOUT_RESET);

	hal_delay_us( 200 );

	// release CS
	//===========
	len   = 4;
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, buff, &len );  
}


//=====================================================
//              NAND::read_id()
//              ---------------
//
// Read NAND Flash ID
//=====================================================
void NAND::read_id( cyg_uint8 id[4] )
{
cyg_uint32 len;
cyg_uint8 *buff = id;

	// assert CS
	//==========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_ASSERT_CS, NULL, NULL );  

	// write ID_READ command
	//======================
	buff[0] = SM_CMD_ID_READ;
	len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_CMD );

	// write ADDR = 0
	//===============
	buff[0] = 0;
	len = 1;
	cyg_io_bwrite( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_ADDR );

	// read the chip ID
	//=================
	len = 4;
	cyg_io_bread( nandio_handle, buff, &len, CYG_IO_CONFIG_FLASH_NAND_READ8 );
    
	// release CS
	//===========
	cyg_io_set_config( nandio_handle, CYG_IO_CONFIG_FLASH_NAND_RELEASE_CS, NULL, NULL );  
    
}

