// ====================================================================
//      					nandio.h
//							--------
//	www.ronetix.at
// ====================================================================
#ifndef CYGONCE_NAND_H
#define CYGONCE_NAND_H

#define SM_CMD_RESET                    0xFF
#define SM_CMD_STATUS_READ              0x70
#define SM_CMD_ID_READ                  0x90
#define SM_CMD_AUTO_BLOCK_ERASE         0x60
#define SM_CMD_AUTO_BLOCK_ERASE2        0xD0
#define SM_CMD_AUTO_PROGRAM             0x10
#define SM_CMD_SERIAL_DATA_INPUT        0x80
#define SM_CMD_READ_MODE1               0x00
#define SM_CMD_READ_MODE2               0x01
#define SM_CMD_READ_MODE3               0x50
#define SM_CMD_READ_SPARE               SM_CMD_READ_MODE3

#define SM_STATUS_PASS_FAIL             0x01        // 0:Pass, 1:Fail
#define SM_STATUS_BUSY                  0x40        // 0:Not Suspended, 1:Suspended
#define SM_STATUS_SUSPEND               0x20        // 0:Not Suspended, 1:Suspended
#define SM_STATUS_WR_PROTECT            0x80        // 0:Protect, 1:Not Protect

#define	NAND_CMD_PAGE_READ1				0x00		// page read cycle 1
#define	NAND_CMD_PAGE_READ2				0x30		// page read cycle 2
#define	NAND_CMD_SERIAL_DATA_INPUT		0x80
#define	NAND_CMD_AUTO_PROGRAM			0x10
#define NAND_CMD_STATUS_READ            0x70
#define NAND_CMD_AUTO_BLOCK_ERASE1      0x60
#define NAND_CMD_AUTO_BLOCK_ERASE2      0xD0


#define SM_TMOUT_PROG                   50          // tPROG : 20ms ----- Program Time
#define SM_TMOUT_ERASE                  800         // tBERASE: 400ms ----- Block Erase Time
#define SM_TMOUT_READ                   80          // tR : 100us ----- Data transfer Time
#define SM_TMOUT_RESET                  60          // tRST : 6ms ----- Device Resetting Time





// Struct Definition
//==================
typedef enum
{
	NAND_MAN_ID_SAMSUNG = 0xEC,
	NAND_MAN_ID_MICRON  = 0x2C,
	NAND_MAN_ID_ST		= 0x20,	
	
} NAND_MAN_ID;

#define	IS_LARGE_PAGE(nn) 		(nn > 526)


typedef struct 
{
	int	 	 	id;
	int			page_size;
	int			spare_size;
	int			pages_per_block;
	int			num_blocks;
	int			n_addr_cycles;
	int			width;
		
} NAND_INFO;


class NAND
{
private:

	char 	*bad_blocks;
	int		num_bad_blocks;
  
	void 	write_addr( cyg_uint32 addr, int offset );
	void 	write_addr_erase(int block_num);
	int 	wait_busy(int timeout);
	void	reset(void);
	void    read_id( cyg_uint8 id[4] );
	int		(*is_busy_asserted)(void);
	void	print_info(void);
	
	
	typedef int (NAND::*PAGE_READ_FUNC)(cyg_uint32 start, cyg_uint8 *buff_data, cyg_uint8 *buff_spare );
	PAGE_READ_FUNC	p_read_page;

    int page_read_small(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare );
    int page_read_large(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare );


	int NAND::look_bad_blocks(void);
	int NAND::is_bad( int page_num );


public:

	cyg_io_handle_t	nandio_handle;

	const NAND_INFO *info;

    int init();
    int page_read(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare );
    int page_write(cyg_uint32 page_num, cyg_uint8 *buff_data, cyg_uint8 *buff_spare );
    int block_erase( int page_num );
    int mark_bad_block( int block_num );

};


#endif
