/*==========================================================================
//
//      at91sam9261_misc.c
//
//      HAL misc board support code for Atmel AT91SAM9261
//
//==========================================================================
//####ECOSGPLCOPYRIGHTBEGIN####
// -------------------------------------------
// This file is part of eCos, the Embedded Configurable Operating System.
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
// Copyright (C) 2003 Nick Garnett <nickg@calivar.com>
// Copyright (C) 2006 eCosCentric Ltd
// Copyright (C) 2006 Andrew Lunn <andrew.lunn@ascom.ch>
//
// eCos is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 or (at your option) any later version.
//
// eCos is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with eCos; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//
// As a special exception, if other files instantiate templates or use macros
// or inline functions from this file, or you compile this file and link it
// with other works to produce a work based on this file, this file does not
// by itself cause the resulting work to be covered by the GNU General Public
// License. However the source code for this file must still be made available
// in accordance with section (3) of the GNU General Public License.
//
// This exception does not invalidate any other reasons why a work based on
// this file might be covered by the GNU General Public License.
// -------------------------------------------
//####ECOSGPLCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):    ilko iliev
// Contributors: gthomas, jskov, nickg, tkoeller, Oliver Munz, Andrew Lunn
// Date:         2006-03-09
// Purpose:      HAL board support
// Description:  Implementations of HAL board interfaces
//
//####DESCRIPTIONEND####
//
//========================================================================*/

#include <pkgconf/system.h>
#include <pkgconf/hal.h>

#include <cyg/infra/cyg_type.h>         // base types
#include <cyg/infra/cyg_trac.h>         // tracing macros
#include <cyg/infra/cyg_ass.h>          // assertion macros

#include <cyg/hal/hal_io.h>             // IO macros
#include <cyg/hal/hal_arch.h>           // Register state info
#include <cyg/hal/hal_diag.h>
#include <cyg/hal/hal_intr.h>           // necessary?
#include <cyg/hal/hal_cache.h>
#include <cyg/hal/hal_if.h>             // calling interface
#include <cyg/hal/hal_misc.h>           // helper functions
#ifdef CYGDBG_HAL_DEBUG_GDB_BREAK_SUPPORT
#include <cyg/hal/drv_api.h>            // HAL ISR support
#endif

// The development board has two LEDs
//===================================
void hal_at91_led (int val)
{
}

void hal_at91_set_leds (int val)
{
    hal_at91_led(val);
}

// -------------------------------------------------------------------------
// Hardware init

void hal_plf_hardware_init (void) 
{

	#define USART0_SIGNALS		( (unsigned int) AT91C_PC9_RXD0 | (unsigned int) AT91C_PC8_TXD0 ) 
										
	// Enable the Serial devices to driver the serial port pins 
  	//=========================================================
	AT91C_BASE_PIOC->PIO_ASR = USART0_SIGNALS;
	AT91C_BASE_PIOC->PIO_PDR = USART0_SIGNALS;	
	
  	// Setup the Reset controller. Allow user resets 
  	//==============================================
	AT91C_BASE_RSTC->RSTC_RMR = 0xA5000000 | AT91C_RSTC_URSTEN | AT91C_RSTC_ERSTL;


#ifdef CYGBLD_HAL_ARM_AT91_SERIAL_UART

	// Enable Peripheral clock in PMC for USART
	//=========================================
	AT91C_BASE_PMC->PMC_PCER = (unsigned int)(1 << AT91C_ID_US0) |
							   (unsigned int)(1 << AT91C_ID_US1) |
							   (unsigned int)(1 << AT91C_ID_SYS ); 
#endif

#ifdef CYGBLD_HAL_ARM_AT91_TIMER_TC


	// Enable peripheral clocks for TC 0 and 1 if they are to be used
  	//===============================================================
  	AT91C_BASE_PMC->PMC_PCER = (unsigned int)(1 << AT91C_ID_TC0) |
							   (unsigned int)(1 << AT91C_ID_TC1) |
							   (unsigned int)(1 << AT91C_ID_TC2);

#endif

#ifndef CYGPKG_IO_WATCHDOG

	// Disable the watchdog. The eCos philosophy is that the watchdog is
    // disabled unless the watchdog driver is used to enable it.
	// Whoever if we disable it here we cannot re-enable it in the
	// watchdog driver, hence the conditional compilation.
	//==================================================================
	AT91C_BASE_WDTC->WDTC_WDMR = AT91C_WDTC_WDDIS;
#endif


}

// Calculate the baud value to be programmed into the serial port baud
// rate generators. This function will determine what the clock speed
// is that is driving the generator so it can be used in situations
// when the application dynamically changes the clock speed. 
cyg_uint32 hal_at91_us_baud(cyg_uint32 baud_rate)
{
cyg_uint32 val;
cyg_uint32 main_clock = 0;
cyg_uint32 baud_value = 0;

  	HAL_READ_UINT32((AT91_PMC+AT91_PMC_MCKR), val);
  	switch (val & 0x03) 
  	{
    	/* Slow clock */
    	case AT91_PMC_MCKR_SLOW_CLK:
      		main_clock = CYGNUM_HAL_ARM_AT91_SLOW_CLOCK;
      		break;
      
      	/* Main clock */
    	case AT91_PMC_MCKR_MAIN_CLK:
      		main_clock = CYGNUM_HAL_ARM_AT91_CLOCK_OSC_MAIN;
      		break;
      		
      	/* PLL */
    	case AT91_PMC_MCKR_PLL_CLK:
//      		HAL_READ_UINT32((AT91_PMC+AT91_PMC_PLLR), pll);
//      		main_clock = CYGNUM_HAL_ARM_AT91_CLOCK_OSC_MAIN * (((pll & 0x7FF0000) >> 16) + 1) / (pll & 0xFF);
				main_clock = CYGNUM_HAL_ARM_AT91_CLOCK_SPEED;
      		break;
  }
  
  // Process prescale
  val = (val & 0x1C) >> 2;
  main_clock = main_clock >> val;

  /* Define the baud rate divisor register, (round) */
  baud_value = (main_clock/(8*baud_rate)+1)/2;
  
  return baud_value;
}


//==================================================================================
//						hardware_init()
//						---------------
//
//===================================================================================
void hardware_init(void)
{
int value;

	// initialite CS2, 16 bit access
	//=============================
	AT91C_BASE_SMC->SMC_SETUP2 = 3*AT91C_SMC_NWESETUP +				// NWE setup length
								 3*AT91C_SMC_NCSSETUPWR +			// NCS setup length in write access
								 3*AT91C_SMC_NRDSETUP + 			// NRD setup length
								 3*AT91C_SMC_NCSSETUPRD;			// NCS setup length in read access
								 
	AT91C_BASE_SMC->SMC_PULSE2 = 4*AT91C_SMC_NWEPULSE +				// NWE pulse length
								 4*AT91C_SMC_NCSPULSEWR +			// NCS pulse length in write access
								 4*AT91C_SMC_NRDPULSE +				// NRD pulse length
								 4*AT91C_SMC_NCSPULSERD;			// NCS pulse length in read access
								 
	AT91C_BASE_SMC->SMC_CYCLE2 = 8*AT91C_SMC_NWECYCLE +				// Total Write cycle length
								 8*AT91C_SMC_NRDCYCLE;				// Total Read cycle length
	
	AT91C_BASE_SMC->SMC_CTRL2 = AT91C_SMC_READMODE +
								AT91C_SMC_WRITEMODE +				
								AT91C_SMC_NWAITM_NWAIT_DISABLE +	// NWAIT disabled	
								AT91C_SMC_DBW_WIDTH_SIXTEEN_BITS +	// 16-bit bus
								6*AT91C_SMC_TDF +					// data float time
								AT91C_SMC_TDFEN;					// TDF optimization enabled
								 								 

	// configure clocks
	// assume:
	//    main osc = 18.432Mhz
	//    PLLB configured for 96MHz (48MHz after div)
	//    CSS = PLLB
	
	// set PLLA = 239MHz
	// assume main osc = 18.432Mhz
	// div = 1 , mul = 13 
	//=====================================
	if ( AT91C_BASE_PMC->PMC_MCKR & AT91C_PMC_CSS )
		return;
	
	// Enable the main oscillator
	//===========================
	AT91C_BASE_CKGR->CKGR_MOR |= AT91C_CKGR_MOSCEN;

	if ( (AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCKA) == 0 )
	{
		value = AT91C_BASE_CKGR->CKGR_PLLAR;
		value &= ~AT91C_CKGR_DIVA;
		value &= ~AT91C_CKGR_OUTA;
		
		value |= (CYGNUM_HAL_ARM_AT91_PLLA_DIVIDER);
		value |= ( CYGNUM_HAL_ARM_AT91_CPU_CLOCK_SPEED > 180 ) ? AT91C_CKGR_OUTA_2 : AT91C_CKGR_OUTA_1;

		value |= AT91C_CKGR_SRCA;
		
		AT91C_BASE_CKGR->CKGR_PLLAR = value;
	
		value = AT91C_BASE_CKGR->CKGR_PLLAR;
		value &= ~AT91C_CKGR_MULA;
		
		value |= ((CYGNUM_HAL_ARM_AT91_PLLA_MULTIPLIER-1) << 16);				// PLL_CLOCK = (CLOCK_IN/DIVIDER)*(MUL+1)
	
		AT91C_BASE_CKGR->CKGR_PLLAR = value;
	
		// wait for lock
		//==============
		while (!( AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCKA) );
	}

	// change divider = 3, pres = 1
	//=============================
	value = AT91C_BASE_PMC->PMC_MCKR;
	value &= ~AT91C_PMC_MDIV;
 	
	value |= AT91C_PMC_MDIV_2;

	value &= ~AT91C_PMC_PRES;
	value |= AT91C_PMC_PRES_CLK;
	AT91C_BASE_PMC->PMC_MCKR = value;

	// wait for update
	//================
	while ( !(AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY) ) ;

	// change CSS = PLLA
	//===================
	value &= ~AT91C_PMC_CSS;
	value |= AT91C_PMC_CSS_PLLA_CLK;
	AT91C_BASE_PMC->PMC_MCKR = value;

	// wait for update
	//================
	while ( !( AT91C_BASE_PMC->PMC_SR & AT91C_PMC_MCKRDY) );



	// USB clock 48MHz
	//================
  	AT91C_BASE_PMC->PMC_SCER = AT91C_PMC_UDP;
	AT91C_BASE_PMC->PMC_PCER = AT91C_ID_UDP;

	if ( (AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCKB) == 0 )
	{
		value = AT91C_BASE_CKGR->CKGR_PLLBR;
		value &= ~AT91C_CKGR_DIVB;
		value &= ~AT91C_CKGR_OUTB;
		
		value |= (CYGNUM_HAL_ARM_AT91_PLLB_DIVIDER);
		value |= ( CYGNUM_HAL_ARM_AT91_PLLB_CLOCK > 180 ) ? AT91C_CKGR_OUTB_2 : AT91C_CKGR_OUTB_1;

		if ( CYGNUM_HAL_ARM_AT91_USB_CLOCK_DIVISION == 1 )
			value |= AT91C_CKGR_USBDIV_0;							// divided by 1	
		else	
		if ( CYGNUM_HAL_ARM_AT91_USB_CLOCK_DIVISION == 2 )
			value |= AT91C_CKGR_USBDIV_1;							// divided by 2
		else	
			value |= AT91C_CKGR_USBDIV_2;							// divided by 4
		
		AT91C_BASE_CKGR->CKGR_PLLBR = value;
	
		// mul = 13
		//=========
		value = AT91C_BASE_CKGR->CKGR_PLLBR;
		value &= ~AT91C_CKGR_MULB;
		
		value |= (12 << 16);				// PLL_CLOCK = (CLOCK_IN/DIVIDER)*(MUL+1)
	
		AT91C_BASE_CKGR->CKGR_PLLBR = value;
	
		// wait for lock
		//==============
		while (!( AT91C_BASE_PMC->PMC_SR & AT91C_PMC_LOCKB) );
	}



	// setup SDRAM access
	// EBI chip-select register (CS1 = SDRAM controller)
	// 9 col, 13row, 4 bank, CAS2
	// write recovery = 2 (Twr)
	// row cycle = 5 (Trc)
	// precharge delay = 2 (Trp)
	// row to col delay 2 (Trcd)
	// active to precharge = 4 (Tras)
	// exit self refresh to active = 6 (Txsr)
	//=========================================

	
	// CS1 is for SDRAMC
	//==================
    AT91C_BASE_MATRIX->MATRIX_EBICSA |= AT91C_MATRIX_CS1A_SDRAMC;

	// Enable Peripheral clock in PMC for  SDRAMC
	//===========================================
	AT91C_BASE_PMC->PMC_PCER = (unsigned int)(1 << AT91C_ID_SYS);
	
	
	#define SDRAMC_SIGNALS			   ((unsigned int) AT91C_PC20_D20      | \
										(unsigned int) AT91C_PC21_D21      | \
										(unsigned int) AT91C_PC30_D30      | \
										(unsigned int) AT91C_PC22_D22      | \
										(unsigned int) AT91C_PC31_D31      | \
										(unsigned int) AT91C_PC23_D23      | \
										(unsigned int) AT91C_PC16_D16      | \
										(unsigned int) AT91C_PC24_D24      | \
										(unsigned int) AT91C_PC17_D17      | \
										(unsigned int) AT91C_PC25_D25      | \
										(unsigned int) AT91C_PC18_D18      | \
										(unsigned int) AT91C_PC26_D26      | \
										(unsigned int) AT91C_PC19_D19      | \
										(unsigned int) AT91C_PC27_D27      | \
										(unsigned int) AT91C_PC28_D28      | \
										(unsigned int) AT91C_PC29_D29      )

	// Configure PIO controllers to drive SDRAMC signals
	//==================================================
	AT91C_BASE_PIOC->PIO_ASR = SDRAMC_SIGNALS;
	AT91C_BASE_PIOC->PIO_BSR = 0;
	AT91C_BASE_PIOC->PIO_PDR = SDRAMC_SIGNALS;

	AT91C_BASE_SDRAMC->SDRAMC_CR =
		AT91C_SDRAMC_NC_9 |
		AT91C_SDRAMC_NR_13 |
		AT91C_SDRAMC_NB_4_BANKS |
		AT91C_SDRAMC_CAS_2 |
		AT91C_SDRAMC_TWR_2 |
		AT91C_SDRAMC_TRC_5 |
		AT91C_SDRAMC_TRP_2 |
		AT91C_SDRAMC_TRCD_2 |
		AT91C_SDRAMC_TRAS_4 |
		AT91C_SDRAMC_TXSR_6;


	AT91C_BASE_SDRAMC->SDRAMC_MR = AT91C_SDRAMC_DBW_32_BITS | AT91C_SDRAMC_MODE_PRCGALL_CMD;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;

	AT91C_BASE_SDRAMC->SDRAMC_MR = AT91C_SDRAMC_DBW_32_BITS | AT91C_SDRAMC_MODE_RFSH_CMD;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;

	AT91C_BASE_SDRAMC->SDRAMC_MR = AT91C_SDRAMC_DBW_32_BITS | AT91C_SDRAMC_MODE_LMR_CMD;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;

	AT91C_BASE_SDRAMC->SDRAMC_TR = 7 * CYGNUM_HAL_ARM_AT91_CLOCK_SPEED / 1000000;

	*(unsigned int*)AT91C_EBI_SDRAM = 0;

	AT91C_BASE_SDRAMC->SDRAMC_MR = AT91C_SDRAMC_DBW_32_BITS | AT91C_SDRAMC_MODE_NORMAL_CMD;
	*(unsigned int*)AT91C_EBI_SDRAM = 0;

	// REMAP
	//======
	AT91C_BASE_MATRIX->MATRIX_MCFG = AT91C_MATRIX_RCA926I | AT91C_MATRIX_RCA926D;
}


// Is DM9000 present?
int cyg_hal_dm9000_present(void) 
{
	return true;
}


//--------------------------------------------------------------------------
// EOF at91sam9261_misc.c
