//==========================================================================
//      						ds2401.c
//								--------
//
//						DS2401 Interface Module
//
//	Exported functions:
//		int ds_init(void);
//		int ds_get_id( void *buffer );
//
//==========================================================================
// Author(s):
//
// Contributors:
// Date:
// Purpose:
// Description: 
//==========================================================================
#include <stdlib.h>
#include <string.h>
#include <cyg/hal/hal_diag.h>

#define DS_PIN				AT91_GPIO_PA7
#define DS_READ_TRIES		10
#define DS_COMMAND_READ_ROM 0x33

#ifndef EINTR
	#define ENOERR 				0
	#define EINTR 				1
	#define EINVAL				2	
#endif

static char DS2401_rom[8];

#define ds_delay 	hal_delay_us


static const cyg_uint8 ds_table[] = 
{
	0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
	157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
	35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
	190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
	70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
	219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
	101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
	248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
	140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147, 205,
	17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178, 236, 14, 80,
	175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143, 12, 82, 176, 238,
	50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76, 18, 145, 207, 45, 115,
	202, 148, 118, 40, 171, 245, 23, 73, 8, 86, 180, 234, 105, 55, 213, 139,
	87, 9, 235, 181, 54, 104, 138, 212, 149, 203, 41, 119, 244, 170, 72, 22,
	233, 183, 85, 11, 136, 214, 52, 106, 43, 117, 151, 201, 74, 20, 246, 168,
	116, 42, 200, 150, 21, 75, 169, 247, 182, 232, 10, 84, 215, 137, 107, 53
};

//============================================================================
//
//
//                   BYTE  CrcDallas( BYTE *src, int len )
//
//                       DALLAS ONE-WIRE CRC
//
//   Parameters:
//        src -> Pointer to array of bytes
//        len -> Length of array (include the CRC sum - last byte)
//
//   Return:
//        0 -> If calculated CRC sum is OK
//        Non zero -> If error is occurred
//
//   Description:
//        Caclulate CRC-8 of given array 'src' with length 'len'
//        Used 8-bit CRC sum with DALLAS fix polynom 0x31
//
//============================================================================
static int  ds_crc( cyg_uint8 *src, int len )
{
int ii;
cyg_uint8 *pp, crc;

	pp = (cyg_uint8 *)ds_table;

	for( ii=crc=0; ii < len; ii++ )
	   crc = *(pp +  (crc ^ src[ii]) );

	return (int)crc;
}


//============================================================================
//
//						  CLEAR2401, SET2401, READ2401
//
// Parameter: 
//		None
//
// Return:
//		None
//
// Description:
//      Interface functions for the PIO
//
//============================================================================
static void CLEAR2401(void)
{
	HAL_ARM_AT91_GPIO_PUT( DS_PIN, false );
}

static void SET2401(void)
{
	HAL_ARM_AT91_GPIO_PUT( DS_PIN, true );
}

static int READ2401(void)
{
int val;
	
	HAL_ARM_AT91_GPIO_GET( DS_PIN, val );	
	
	return val;
}

//============================================================================
//
//                          int ds_reset(void)
//
// Parameter: None
//
// Return:
//      ENOERR  -> If no error
//      EINTR   -> If not 'presence pulse' is detected
//
// Description:
//      Make initialization procedure "Reset and Presence Pulses"
//
//============================================================================
#define tRSTL           600        // 480uS < tRSTL < 960uS
#define tRSTH           600        // 480uS < tRSTH < 960uS
#define tPDHmin          10        // 15uS
#define tPDHmax         100        // 60uS
#define tPDLmin          50        // 60uS   
#define tPDLmax         250        // 240uS   
#define tREC             10
static int ds_reset(void)
{

	// Reset pulse
	//============
	CLEAR2401();
	ds_delay( tRSTL ); 
	SET2401();

	ds_delay( tRSTH );   

	// Return 0 if no error is occurred
	// ================================
	return ENOERR;
}

//============================================================================
//
//                   void write_one(void)
//
// Parameter: None
//
// Return: None
//
// Description:
//      Make Write-One Time Slot
//
//============================================================================
#define tLOW1           2
#define tSLOT           60

static void ds_write_one(void)
{
	CLEAR2401();
	ds_delay( tLOW1 );
	SET2401();
	ds_delay( tSLOT - tLOW1 );  
	ds_delay( tREC );  
}

//============================================================================
//
//                         void write_zero(void)
//
// Parameter: None
//
// Return: None
//
// Description:
//     Make Write-Zero Time Slot
//
//============================================================================
static void ds_write_zero(void)
{
	CLEAR2401();
	ds_delay( tSLOT );
	SET2401();
	ds_delay( tREC );    
}

//============================================================================
//
//                      int ReadData( void )
//
// Parameter: None
//
// Return: None
//
// Description: Make Read-data Time Slot
//
//============================================================================
#define tLOWR             2 
#define tRDV             15
#define tRELEASE         30
static int ds_read_data(void)
{
int ret;

  
	CLEAR2401();
	ds_delay( tLOWR );        
	SET2401();  

	ds_delay( 1 );          

	HAL_ARM_AT91_GPIO_CFG_DIRECTION( DS_PIN, AT91_PIN_IN);	

	ret = READ2401();

	HAL_ARM_AT91_GPIO_CFG_DIRECTION( DS_PIN, AT91_PIN_OUT);	
	HAL_ARM_AT91_GPIO_CFG_MULTI( DS_PIN, true );


	ds_delay( tRELEASE + tREC );    

	return ret;
}

//============================================================================
//
//                             void ds_send_byte(BYTE ch)
//
//
// Parameter:
//      ch  ->  byte to send
//
// Returns: None
//
// Description:
//      Send a byte to DS2401
//
//============================================================================
static void ds_send_byte( cyg_uint8 ch )
{
int ii, mask;
  
	for( ii=0, mask = 0x01; ii < 8; ii++, mask <<= 1 ) 
		if( ch & mask ) 
			ds_write_one();
		else
            ds_write_zero();
}

//============================================================================
//
//                                  ds_read_byte
//
// Parameter:
//      None
//
// Return:
//      The data read from DS2401
//
// Description:
//      Read a byte from DS2401
//
//============================================================================
static cyg_uint8 ds_read_byte(void)
{
int ii, mask;
cyg_uint8 ret;
  
	for( ret=0,ii=0, mask = 0x01; ii < 8; ii++, mask <<= 1 )
		if( ds_read_data() ) 
			ret |= mask;
       
	return ret;     
}

//=====================================================================
//                      ds_init()
//                      ----------
//
//		Read DS2401 module and fill the array
//
//=====================================================================
int ds_init(void)
{
static const unsigned char fix_mac[8] = { 0xAA, 0xBB, 0xCC, 0xDD, 0x12, 0x34, 0x56, 0x78 };
int ii, tries, crc;
int sum, val;

	// Enable Peripheral clock in PMC for PIOA
	//========================================
	AT91C_BASE_PMC->PMC_PCER = (unsigned int)(1 << AT91C_ID_PIOA);

	// Prepare PIO pin
	//================
	HAL_ARM_AT91_GPIO_PUT( DS_PIN, true );
	HAL_ARM_AT91_GPIO_CFG_DIRECTION( DS_PIN, AT91_PIN_OUT);	
	HAL_ARM_AT91_GPIO_CFG_MULTI( DS_PIN, true );
	

	for ( tries=0; tries<DS_READ_TRIES; tries++ )
	{
		// Reset the DS2401
		// ================
		if ( ds_reset() )
			continue;

		// Send command READ ROM
		// =====================
		ds_send_byte( DS_COMMAND_READ_ROM );

		// Read 64-bit unique number
		// =========================
		for( sum=ii=0; ii < 8; ii++ ) 
		{
			val = ds_read_byte();
			DS2401_rom[ii] = val;
			sum += val;
			ds_delay(200);
		}
		
		if ( sum == 0 )
			continue;
		
		crc = ds_crc( DS2401_rom, sizeof(DS2401_rom)-1 );
		if ( crc == DS2401_rom[sizeof(DS2401_rom)-1] )
			break;
	}

	if ( tries == DS_READ_TRIES )
	{
	    memset( DS2401_rom, fix_mac, sizeof(DS2401_rom) );
		return EINTR;
	}

    return ENOERR;
}

//===================================================================================================
//                              ds_get_config()
//                              --------------
//  Parameter:
//      handle -> channel handle
//      key    -> I2C_CONFIG_PRESCALER to get the REG_IICPS
//      buffer -> pointer to configuration data
//      len    -> pointer to variable that contain the size of buffer. MUST exactly the size of
//                parameter's structure
//
//  Rerturn:
//      ENOERR - if no error
//      EINVAL - if the size of buffer don't mach the size of parameter's structure
//
//  Description:
//      Return in 'buffer' the parameter's structure of channel
//===================================================================================================
int ds_get_id( void *buffer, int buff_size )
{
	if ( buff_size < sizeof(DS2401_rom) )
		return EINVAL;
	
    memcpy( buffer, DS2401_rom, sizeof(DS2401_rom) );
    return ENOERR;
}



int ds_get_mac( unsigned char mac[6] )
{
int ii;
unsigned char buff[8];

#define SWAP( A, B ) { buff[0]=A; A=B; B=buff[0]; }

	// Ask for the ID
	//===============
	ds_get_id( buff, sizeof(buff) );

	// Make sure it is OK
	//===================
	for ( ii=7; ii>=0; ii-- )
		if ( buff[ii] )
			break;

	if ( ii < 0 ) return true;

	memmove( mac, buff+1, 6 );

	SWAP( mac[0], mac[5] );
	SWAP( mac[1], mac[4] );
	SWAP( mac[2], mac[3] );

	mac[0] = 0x02;

	return false;
}
