//=============================================================================
//							msd.h
//							-----
//	
//=============================================================================


// Table 2.1 - SubClassCodes Mapped to Command Block Specifications, usb_msc_overview_1.2.pdf
//===========================================================================================
#define USB_DEVICE_DESCRIPTOR_CLASS_MASS_STORAGE       8
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_RBC     		0x01
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_MCC 			0x02
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_QIC     		0x03
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_UFI     		0x04
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_SFF     		0x05
#define USB_DEVICE_DESCRIPTOR_SUBCLASS_SCSI    		0x06


// Table 3.1 - Mass Storage Transport Protocol, usb_msc_overview_1.2.pdf
//======================================================================
#define USB_INTERFACE_DESCRIPTOR_PROTOCOL_CRTL_BULK_IT_WITH_COMPL 		0x00
#define USB_INTERFACE_DESCRIPTOR_PROTOCOL_CRTL_BULK_IT_NO_COMPL   		0x01
#define USB_INTERFACE_DESCRIPTOR_PROTOCOL_BULK_ONLY_TRANSPORT     		0x50

// *********************************************
// usbmassbulk_10.pdf : Class Specific Requests
// *********************************************

// Table 5.3 - Command Block Status Values 
//========================================
typedef enum
{
	GOOD_STATUS,         // 00h Command Passed ("good status") 
  	COMMAND_FAILED,      // 01h Command Failed 
  	PHASE_ERROR          // 01h Command Failed 
  
} COMMAND_BLOCK_STATUS;

#define VALUE_32(p1,p2,p3,p4) ((p1)|(p2<<8)|(p3<<16)|(p4<<24))


// 3.1 Bulk-Only Mass Storage Reset (class-specific request)
// This request is used to reset the mass storage device and its associated interface.
#define RESET_REQUEST_TYPE   0x21  // bmRequestType
#define RESET_REQUEST        0xFF  // bRequest
// wValue field set to 0
// wIndex field set to the interface number
// wLength field set to 0


// 3.2 Get Max LUN (class-specific request)
#define MAX_LUN_REQUEST_TYPE 0xA1  // bmRequestType
#define MAX_LUN_REQUEST      0xFE  // bRequest
// wValue field set to 0
// wIndex field set to the interface
// wLength field set to 1

#define MAX_LUN         0			
#define MAX_LUN_LENGTH  1
#define BLOCKLEN_512    0x0200



#define CBW_SIZE         31
#define CBW_SIGNATURE    0x43425355

typedef struct _USB_MSD_CBW 				// 31 bytes total Command Block Wrapper (little endian)
{
    cyg_uint8 dCBWSignature[4];				// 55 53 42 43 hex
    cyg_uint8 dCBWTag[4];					// sent by host, device echos this value in CSW (associated a CSW with a CBW)
    cyg_uint8 dCBWDataTransferLength[4]; 	// number of bytes of data host expects to transfer
    cyg_uint8 bCBWFlags; 					// CBW flags, bit 7 = 0-data out from host to device,  1-device to host, rest bits 0
    cyg_uint8 bCBWLUN;						// Most Significant 4bits are always zero, 0 in our case as only one logical unit
    cyg_uint8 bCBWCBLength;					// Here most significant 3bits are zero
	cyg_uint8 CBWCB[16];		        	// Command block to be executed by the device
} USB_MSD_CBW;

// Table 5.2 Command Status Wrapper (CSW) 
//=======================================
#define CSW_SIZE         13
#define CSW_SIGNATURE    0x53425355
typedef struct _USB_MSD_CSW				// Command Status Wrapper (13 bytes)
{
	cyg_uint8 dCSWSignature[4];			// 55 53 42 53 hex Signature of a CSW packet
	cyg_uint8 dCSWTag[4];				// echo the dCBWTag of the CBW packet
	cyg_uint8 dCSWDataResidue[4];		// difference in data expected (dCBWDataTransferLength) and actual amount processed/sent
	cyg_uint8  bCSWStatus;				// 00h Command Passed, 01h Command Failed, 02h Phase Error, rest obsolete/reserved
	
} USB_MSD_CSW;




// GENERIC_RBC : Structure representing the generic RBC command 
//=============================================================
typedef struct
{
  cyg_uint8 operation_code;   		// operation code of the rbc request 
  cyg_uint8 reserved[8];      		// Reserved; must be set to 0        
  cyg_uint8 control;          		// control code; must be set to zero 
  
} GENERIC_RBC;


// USBMS_READ_10 : Table 4 - READ (10) Command Descriptor Block, rbc-10a.pdf 
//==========================================================================
typedef struct
{
  cyg_uint8 operation_code;   /* Operation code : 0x28 : RBC_READ_10 */
  cyg_uint8 reserved_1;       /* reserved; must not be used          */
  cyg_uint8 block_address_3;  /* Logical Block Address MSB           */
  cyg_uint8 block_address_2;  /* Logical Block Address               */
  cyg_uint8 block_address_1;  /* Logical Block Address               */
  cyg_uint8 block_address_0;  /* Logical Block Address LSB           */
  cyg_uint8 reserved_2;       /* reserved; must not be used          */
  cyg_uint8 xfer_len_1;       /* Transfer Length MSB                 */
  cyg_uint8 xfer_len_0;       /* Transfer Length LSB                 */
  cyg_uint8 control;          /* control code : 0x00                 */
  
} USBMS_READ_10;

// USBMS_READ_CAPACITY : Table 5 - READ CAPACITY Command Descriptor Block, rbc-10a.pdf 
//====================================================================================
typedef struct
{
  cyg_uint8 operation_code;     /* Operation code : 0x25 : RBC_READ_CAPACITY */
  cyg_uint8 reserved[8];        /* Must not be used                          */
  cyg_uint8 control;            /* Control code : 0x00                       */
  
} USBMS_READ_CAPACITY;


// START / STOP UNIT RBC command definition.(Not used at this time) 
// Table 7 - START STOP UNIT Command Descriptor Block, rbc-r10a.pdf 
//=================================================================
typedef struct 
{
	cyg_uint8 Operation_Code; /* Operation code : 0x1B : START_STOP_UNIT */
	cyg_uint8 Immediate :1;   /* An IMMEDIATE (IMMED) bit of zero indicates status returned after operation completed */
	cyg_uint8 Reserved1 :7;
	cyg_uint8 Reserved2[2];
	union START_STOP_FLAGS
	{
		struct
		{
		  cyg_uint8 Start            :1;
		  cyg_uint8 Load_Eject       :1;
		  cyg_uint8 Reserved3        :2;
		  cyg_uint8 Power_Conditions :4;
		} bits0;
		
		struct
		{
		  cyg_uint8 Medium_State     :2;
		  cyg_uint8 Reserved3        :2;
		  cyg_uint8 Power_Conditions :4;
		} bits1;
		
	} Flags;
	cyg_uint8 Control;
	
} USBMS_START_STOP_UNIT;


// USBMS_WRITE_10 : Table 11 - WRITE(10) Command Descriptor Block, rbc-10a.pdf 
//============================================================================
typedef struct
{
  cyg_uint8 operation_code;     /* Operation code : 0x2A : RBC_WRITE_10 */
  cyg_uint8 reserved_1:3;       /* reserved                   */
  cyg_uint8 FUA:1;              /* Write to cache not handled */
  cyg_uint8 reserved_2:4;       /* reserved                   */
  cyg_uint8 block_address_3;    /* Logical Block Address MSB  */
  cyg_uint8 block_address_2;    /* Logical Block Address      */
  cyg_uint8 block_address_1;    /* Logical Block Address      */
  cyg_uint8 block_address_0;    /* Logical Block Address LSB  */
  cyg_uint8 reserved_3;         /* Must not be used           */
  cyg_uint8 xfer_len_1;         /* Transfer Length MSB        */
  cyg_uint8 xfer_len_0;         /* Transfer Length LSB        */
  cyg_uint8 control;            /* Control code : 0x00        */
  
} USBMS_WRITE_10;

// USBMS_INQUIRY : Table 75 - INQUIRY command, spc3r17.pdf 
//========================================================
typedef struct
{
  cyg_uint8 operation_code;  /* Operation code : 0x12 : RBC_INQUIRY */
  cyg_uint8 enable_vpd:1;    /* if set to 0, USB-MS shall return the standard inquiry data */
  cyg_uint8 reserved_0:7;    /* Must not be used  */
  cyg_uint8 page_code;       /* specifies which vital product data shall be returned  */
  cyg_uint8 length_1;        /* first block to be verified MSB */
  cyg_uint8 length_0;        /* first block to be verified LSB */
  cyg_uint8 control;         /* Must be set to 0  */
  
} USBMS_INQUIRY;


// USBMS_MODE_SELECT : Table 17 - MODE SELECT(6) Command Descriptor Block, rbc-10a.pdf
//====================================================================================
typedef struct
{
  cyg_uint8 operation_code;        /* Operation code : 0x15 : RBC_MODE_SELECT     */
  cyg_uint8 sp:1;                  /* Save Pages (SP) bit support: 0 for removable*/
  cyg_uint8 reserved_0:3;          /* Must not be used  */
  cyg_uint8 pf:1;                  /* Page Format (PF) : 0x01 */
  cyg_uint8 reserved_1:3;          /* Must not be used  */
  cyg_uint8 reserved_2[2];         /* Must not be used  */
  cyg_uint8 parameter_list_length; /* */
  cyg_uint8 control;               /* Control code : 0x00  */
  
} USBMS_MODE_SELECT;


// USBMS_MODE_SENSE : Table 18 - MODE SENSE(6) Command Descriptor Block, rbc-10a.pdf 
//==================================================================================
typedef struct
{
  cyg_uint8 operation_code;     /* Operation code : 0x1A : RBC_MODE_SENSE */
  cyg_uint8 reserved_0:3;       /* Must not be used                       */
  cyg_uint8 dbd:1;              /* Disable Block Descriptors : 0x01       */
  cyg_uint8 reserved:4;         /* Must not be used     */
  cyg_uint8 page_code:6;        /* Page Code            */
  cyg_uint8 pc:2;               /* Page Control         */
  cyg_uint8 reserved1;
  cyg_uint8 allocation_length;  /* allocation length    */
  cyg_uint8 control;            /* control code : 0x00  */
  
} USBMS_MODE_SENSE;


// USBMS_TEST_UNIT_READY : Table 164 - TEST UNIT READY command, spc3r17.pdf 
//=========================================================================
typedef struct
{
  cyg_uint8 operation_code; 		// Operation code : 0x00 : RBC_TEST_UNIT_READY
  cyg_uint8 reserved[4];
  cyg_uint8 control;
  
} USBMS_TEST_UNIT_READY;


// USBMS_BOT_CDB : Structure for treatment of RBC command
//=======================================================
typedef union
{
  GENERIC_RBC           generic_rbc;
  USBMS_READ_10         read;
  USBMS_READ_CAPACITY   read_capacity;
  USBMS_START_STOP_UNIT start_stop; 
  USBMS_WRITE_10        write;
  USBMS_INQUIRY         inquiry;
  USBMS_MODE_SELECT     mode_select;
  USBMS_MODE_SENSE      mode_sense;
  USBMS_TEST_UNIT_READY test_unit_ready;
  
} USBMS_BOT_CDB;




// 5.8.2. Mode Parameter header (6), rbc-10a.pdf 
//==============================================
typedef struct
{
  cyg_uint8 mode_data_length;          	// if used with MODE_SENSE this field contains the size of the data to transfer excluding this field  
  cyg_uint8 medium_type;               	// medium type : 0x00  
  cyg_uint8 device_specific_parameter; 	// device specific parameter : 0x00  
  cyg_uint8 block_descriptor_length;   	// set to 0  
  
} USBMS_MODE_PARAMETER_HEAD;


// Table 14 - RBC Device Parameters page format, rbc-10a.pdf 
//==========================================================
typedef struct
{
  cyg_uint8 pageCode:6;               /* bit 0->5 : Page code 0x06 */
  cyg_uint8 reserved1:1;              /* bit 6    : reserved       */
  cyg_uint8 PageSavable:1;            /* bit 7    : PS PageSavable */
  cyg_uint8 pageLength;               /* 0x0B          */
  cyg_uint8 writeCacheDisable:1;      /* only on bit 0 */
  cyg_uint8 reserved2:7;
  cyg_uint8 logicalBlockSize[2];      /* MSB - LSB */
  cyg_uint8 numberOfLogicalBlocks[5]; /* MSB - LSB */
  cyg_uint8 powerPerformance;
  cyg_uint8 lockd:1;                  /* bit 0: lockable    */
  cyg_uint8 formatd:1;                /* bit 1: formattable */
  cyg_uint8 writed:1;                 /* bit 2: writable    */
  cyg_uint8 readd:1;                  /* bit 3: readable    */
  cyg_uint8 reserved3:4;
  cyg_uint8 reserved4;
  
} MODE_RBC_DEVICE_PARAMETER_PAGE;

// USBMS_MODE_PARAMETER_HEAD : Table 13 - Mode parameter list, rbc-10a.pdf 
//=========================================================================
typedef struct
{
  USBMS_MODE_PARAMETER_HEAD      head;
  MODE_RBC_DEVICE_PARAMETER_PAGE page;
  
} MODE_SENSE_PARAMETER;




/*
typedef union 
{
	struct
    {
        cyg_uint8 _byte[18];
    };
    
	struct 
	{
		cyg_uint8 ResponseCode:7;			// b6-b0 is Response Code Fixed or descriptor format
		cyg_uint8 VALID:1;					// Set to 1 to indicate information field is a valid value
	
		cyg_uint8 Obsolete;
	
		cyg_uint8 SenseKey:4;				// Refer SPC-3 Section 4.5.6
		cyg_uint8 Resv:1;					
		cyg_uint8 ILI:1;					// Incorrect Length Indicator
		cyg_uint8 EOM:1;					// End of Medium
		cyg_uint8 FILEMARK:1; 				// for READ and SPACE commands
	
		UDWORD 		Information;			// device type or command specific (SPC-33.1.18)
		cyg_uint8 	AddSenseLen;			// number of additional sense bytes that follow <=244
		UDWORD 		CmdSpecificInfo;		// depends on command on which exception occured
		cyg_uint8 	ASC;					// additional sense code 
		cyg_uint8 	ASCQ;					// additional sense code qualifier Section 4.5.2.1 SPC-3
		cyg_uint8 	FRUC;					// Field Replaceable Unit Code 4.5.2.5 SPC-3
	
		cyg_uint8 SenseKeySpecific[3];		// msb is SKSV sense-key specific valied field set=> valid SKS
											// 18-n additional sense bytes can be defined later
											// 18 Bytes Request Sense Fixed Format
	};
	
} REQ_SENSE_RESP;
*/

typedef struct 
{
 	cyg_uint8 Peripheral; 					// Peripheral_Qualifier:3; Peripheral_DevType:5;
	cyg_uint8 Removble;						// removable medium bit7 = 0 means non removable, rest reserved
	cyg_uint8 Version;						// version
	cyg_uint8 Response_Data_Format;		    // b7,b6 Obsolete, b5 Access control co-ordinator, b4 hierarchical addressing support 
										// b3:0 response data format 2 indicates response is in format defined by spec
	cyg_uint8 AdditionalLength;				// length in bytes of remaining in standard inquiry data
	cyg_uint8 Sccstp; 						// b7 SCCS, b6 ACC, b5-b4 TGPS, b3 3PC, b2-b1 Reserved, b0 Protected 
	cyg_uint8 bqueetc;						// b7 bque, b6- EncServ, b5-VS, b4-MultiP, b3-MChngr, b2-b1 Obsolete, b0-Addr16	
    cyg_uint8 CmdQue;                        // b7-b6 Obsolete, b5-WBUS, b4-Sync, b3-Linked, b2 Obsolete,b1 Cmdque, b0-VS
	char vendorID[8];	
	char productID[16];
	char productRev[4];
} InquiryResponse;


// SCSI Transparent Command Set Sub-class code
// ===========================================
#define SCSI_INQUIRY 							0x12
#define SCSI_READ_FORMAT_CAPACITY 				0x23			 
#define SCSI_READ_CAPACITY 						0x25
#define SCSI_READ_10 							0x28
#define SCSI_WRITE_10 							0x2A
#define SCSI_REQUEST_SENSE 						0x03
#define SCSI_MODE_SELECT 						0x15
#define SCSI_MODE_SENSE 						0x1A
#define SCSI_PREVENT_ALLOW_MEDIUM_REMOVAL 		0x1E
#define SCSI_TEST_UNIT_READY 					0x00
#define SCSI_VERIFY 							0x2F
#define SCSI_STOP_START 						0x1B


// Sense Key Error Codes
// =====================
#define S_NO_SENSE          0x00
#define S_RECOVERED_ERROR   0x01
#define S_NOT_READY         0x02
#define S_MEDIUM_ERROR      0x03
#define S_HARDWARE_ERROR    0X04
#define S_ILLEGAL_REQUEST   0x05
#define S_UNIT_ATTENTION    0x06
#define S_DATA_PROTECT      0x07
#define S_BLANK_CHECK       0x08
#define S_VENDOR_SPECIFIC   0x09
#define S_COPY_ABORTED      0x0a
#define S_ABORTED_COMMAND   0x0b
#define S_OBSOLETE          0x0c
#define S_VOLUME_OVERFLOW   0x0d
#define S_MISCOMPARE        0x0e
#define S_CURRENT           0x70
#define S_DEFERRED          0x71

// ASC ASCQ Codes for Sense Data (only those that we plan to use)
// with sense key Illegal request for a command not supported
#define ASC_INVALID_COMMAND_OPCODE                          0x20
#define ASCQ_INVALID_COMMAND_OPCODE                         0x00

// from SPC-3 Table 185
// with sense key Illegal Request for test unit ready
#define ASC_LOGICAL_UNIT_NOT_SUPPORTED                      0x25
#define ASCQ_LOGICAL_UNIT_NOT_SUPPORTED                     0x00

// with sense key Not ready
#define ASC_LOGICAL_UNIT_DOES_NOT_RESPOND                   0x05
#define ASCQ_LOGICAL_UNIT_DOES_NOT_RESPOND                  0x00

#define ASC_MEDIUM_NOT_PRESENT                              0x3a
#define ASCQ_MEDIUM_NOT_PRESENT                             0x00

#define ASC_LOGICAL_UNIT_NOT_READY_CAUSE_NOT_REPORTABLE     0x04
#define ASCQ_LOGICAL_UNIT_NOT_READY_CAUSE_NOT_REPORTABLE    0x00

#define ASC_LOGICAL_UNIT_IN_PROCESS                         0x04
#define ASCQ_LOGICAL_UNIT_IN_PROCESS                        0x01

#define ASC_LOGICAL_UNIT_NOT_READY_INIT_REQD                0x04
#define ASCQ_LOGICAL_UNIT_NOT_READY_INIT_REQD               0x02

#define ASC_LOGICAL_UNIT_NOT_READY_INTERVENTION_REQD        0x04
#define ASCQ_LOGICAL_UNIT_NOT_READY_INTERVENTION_REQD       0x03

#define ASC_LOGICAL_UNIT_NOT_READY_FORMATTING               0x04
#define ASCQ_LOGICAL_UNIT_NOT_READY_FORMATTING              0x04

#define ASC_LOGICAL_BLOCK_ADDRESS_OUT_OF_RANGE              0x21
#define ASCQ_LOGICAL_BLOCK_ADDRESS_OUT_OF_RANGE             0x00

#define ASC_WRITE_PROTECTED                                 0x27
#define ASCQ_WRITE_PROTECTED                                0x00



#define MSD_MAX_MMC_PAGE		2048


typedef enum
{
	USB_MSD_STATE_WAIT,				// Waiting for a valid Command Block Wrapper (CBW)
	USB_MSD_STATE_DATA_IN,			// IN Data State ( Device-> Host)
	USB_MSD_STATE_DATA_OUT,			// OUT Data State (Host -> Device)
	USB_MSD_STATE_DATA_IN_END
	
} USB_MSD_STATE;

typedef struct
{
	int						flag_busy;
	USB_MSD_CONF			conf;
	int						curr_lun;
	
	cyg_io_handle_t 		mmc_handle;
	int						mmc_page_size;
	int						mmc_num_pages;

	cyg_sem_t 				completion_wait;
	
	USB_MSD_STATE			msd_state;
	
	// used form read_10() and write_10()
	//===================================
	cyg_uint32				addr;
	int						len;
	
	cyg_thread 				task_thread_s;		
	char 					task_stack[8*1024];		// space stacks
	cyg_handle_t 			task_thread;

	USB_MSD_CBW 			msd_cbw;           		// msd Command Block Wrapper
	USB_MSD_CBW 			msd_cbw_copy;      		// copy of Command Block Wrapper	  
	USB_MSD_CSW 			msd_csw;                // msd Command Status Wrapper 

	cyg_uint8 				sense_data[18];

	cyg_uint32 				msd_buffer_mem[MSD_MAX_MMC_PAGE/4];
	cyg_uint8 				*msd_buffer;

	
	int	result;
	
} MSD_CHAN;


