//=======================================================================
//					main.c
//					------
//
//	An eCos example for AT91SAM9261 
//
//  CPU  : Atmel At91SAM9261
//	Board: Development Kit EB9261 from www.ronetix.com
//
//  30-04-2007 - Updated
//	30-08-2006 - Created by Ilko Iliev (iliev@ronetix.at)
//
//	www.ronetix.com
//=======================================================================

#include <ctype.h>
#include <network.h>
#include <ftpclient.h>
#include <cyg/kernel/kapi.h>
#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <cyg/fs/fatfs.h>
#include <cyg/fileio/fileio.h>
#include <cyg/io/user_config_keys.h>
#include <cyg/io/io.h>
#include <cyg/io/serialio.h>
#include <assert.h>


extern int errno;

#define PREFIX_ERR		"-- error: "
#define PREFIX_INFO		"++ info: "
#define PREFIX_WARN		"-- warning: "
#define	PREFIX_TEST		"\r\n-> TEST: "

#define ERROR_MSG(format, rest...) 	printf( PREFIX_ERR""__FILE__", %s(), line %d : "format"\r\n", __FUNCTION__,__LINE__, ## rest)


extern cyg_uint8  _binary_rootfs_bin_start;
extern cyg_uint32 _binary_rootfs_bin_size;

MTAB_ENTRY( romfs_mte1,
                   "/romfs",
                   "romfs",
                   "",
                   (CYG_ADDRWORD) &_binary_rootfs_bin_start );


static int print_dir( char *start_dir );

//=======================================================================
//						test_romfs()
//						------------
//	
//	Test the ROM filesystem
//=======================================================================
static void test_romfs(void)
{
	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);
	
	print_dir("/romfs");
}

//=======================================================================
//						test_nand_yaffs()
//						-----------------
//	
//	Test the NAND Flash driver + YAFFS2 Filesystem
//=======================================================================
#ifdef CYGPKG_DEVS_FLASH_NAND_ARM_EB9261
static void test_nand_yaffs(void)
{
int flag_format = 0;	
int stat;
int file_handle;

	printf(PREFIX_TEST"calling %s(flag_format=%d)\r\n", __FUNCTION__, flag_format );

	// mount a partition with start block 1 and end block 500
	// without "start" and "end" parameter the whole size will be used
	//=======================================================
	if ( flag_format )
		stat = mount("/dev/nand0 format=1", "/yaffs", "yaffs" );		// with format 
	else	
		stat = mount("/dev/nand0", "/yaffs", "yaffs" );					// without format	
		
    if( stat < 0 ) 
    	ERROR_MSG( "unable to mount NAND Flash\r\n" );    


	print_dir( "/yaffs" );

	file_handle = open( "/yaffs/test.bin", O_WRONLY|O_CREAT );
	if ( file_handle < 0 ) 
	{
		ERROR_MSG("unable to open file" );
		return;
	}
	
	write( file_handle, "www.ronetix.at",  14);
	
	close( file_handle );	
}

#else

static void test_nand_yaffs(void)
{
	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__ );
	printf(PREFIX_INFO"The NAND Flash driver is not installed\r\n");
	printf(PREFIX_INFO"Please contact www.ronetix.com for more information\r\n");
}

#endif

//=======================================================================
//						test_mmc()
//						----------
//	
//	Test the MMC/SD card driver and the FAT filesystem
//=======================================================================
static int test_mmc(void)
{
	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);

#ifdef CYGPKG_DEVS_DISK_AT91_MCI
#define MMC_MOUNTPOINT				"/dos"	
int stat;



	// try to mount the MMC card - FAT formatted with partition table
	//===============================================================
	stat = mount("/dev/mmc/1", MMC_MOUNTPOINT, "fatfs" );
	if ( stat )
	{
		// try to mount the MMC card - FAT formatted without partition table
		//==================================================================
		stat = mount("/dev/mmc", MMC_MOUNTPOINT, "fatfs" );
		if ( stat )
		{
			ERROR_MSG("unable to mount media, errno=%d -> %s\r\n", errno, strerror(errno) );
			return EIO;
		}
	}
		
	printf(PREFIX_INFO"MMC/SD card mount\r\n");
	
	if ( stat )
	{
		ERROR_MSG("unable to mount media, errno=%d -> %s\r\n", errno, strerror(errno) );
		return EIO;
	}

	print_dir( MMC_MOUNTPOINT );
	
#else

	printf(PREFIX_INFO"for this test the eCos must be compiled with:\r\n" );
	printf("  make ecc_mci\r\n" );
	printf("  make\r\n\n" );

#endif

	return ENOERR;
}

//=======================================================================
//						test_dataflash()
//						----------------
//	Test the DataFlash
//=======================================================================
static void test_dataflash(void)
{
	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);

#ifdef CYGPKG_DEVS_SPI_ARM_AT91
	
int stat, ii, err;	
cyg_uint32 chip_size, page_size, len, *pp;
static char *buff = NULL;
cyg_io_handle_t handle;

	stat = cyg_io_lookup("/dev/at45_dataflash0", &handle);
	if ( stat ) 
	{
		ERROR_MSG("unable to find DataFlash chip\r\n");
		return;
	}

	// get some DataFlash parameters
	//==============================
	len = 4;
	cyg_io_get_config( handle, CYG_IO_CONFIG_SIZE, &chip_size, &len );
	
	len = 4;
	cyg_io_get_config( handle, CYG_IO_CONFIG_BLOCK_SIZE, &page_size, &len );
	
	printf("++ info: DatFlash found:\r\n");
	printf("++ info:   chip size = %d KB\r\n", chip_size/1024);
	printf("++ info:   page size = %d bytes\r\n", page_size);

	// reduce the test buffer size
	//============================
	chip_size /= 64;

	if ( buff == NULL )
	{
		buff = malloc( chip_size );
		if ( buff == NULL )
		{
			ERROR_MSG("not enougth memory\r\n");
			return;
		}
	}

	// fill the buffer with test pattern
	//==================================	
	pp = (cyg_uint32 *)buff;
	for ( ii=0; ii < chip_size/4; ii++ )
		*pp++ = ii;	

	// write test pattern
	//===================	
	len = chip_size / page_size;
	stat = cyg_io_bwrite( handle, buff, &len, 0 );
	if ( stat ) return;

	memset( buff, 0, chip_size );

	// read from DataFlash
	//====================
	stat = cyg_io_bread(handle, buff, &len, 0);	
	if ( stat ) return;

	pp = (cyg_uint32 *)buff;
	for (err=ii=0; ii < chip_size/4; ii++)
		if ( *pp++ != ii )
			err++;

	if ( err == 0 )
		printf(PREFIX_INFO"successfully written and read %d KB\r\n", chip_size/1024);
	else
		ERROR_MSG("%d error(s) found\r\n", err );	
		
#else
	
	printf("\r\n");
	printf(PREFIX_INFO"for this test the eCos must be compiled with:\r\n" );
	printf("  make ecc\r\n" );
	printf("  make\r\n\n" );

#endif		
		
}

//=======================================================================
//						ftpclient_dummy()
//						-----------------
// Used only by ftp_get()
//=======================================================================
static void ftpclient_dummy( unsigned error, const char *fmt, ... )
{
}

//=======================================================================
//						test_net()
//						----------
//	Initialize the network interface and download a file from a FTP server
//=======================================================================
static void test_net(void)
{
static int flag_init_done = false;
int stat;
static char *buff = NULL;
#define NET_BUFF_SIZE			(1500*1024)

	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);

	if ( flag_init_done == false )
	{
		buff = malloc(NET_BUFF_SIZE);
		if ( buff == NULL ) 
		{
			ERROR_MSG("not enougth memory\r\n");
			return;
		}

		init_all_network_interfaces();
		flag_init_done = true;
		
	}

	stat = ftp_get( "192.168.3.1", "user", "password", "uImage", buff,  NET_BUFF_SIZE, ftpclient_dummy);	
	if ( stat > 0 )
		printf(PREFIX_INFO"received via FTP %d bytes\r\n", stat );
	else
		ERROR_MSG("%s\r\n", strerror( errno ) );	
	
}

#ifdef CYGPKG_IO_USB_SLAVE_MSD

#include <cyg/io/usb/usb_msd.h>
extern usbs_control_endpoint usbs_at91_ep0;
extern usbs_rx_endpoint usbs_at91_ep1;          // BULK OUT 
extern usbs_tx_endpoint usbs_at91_ep2;          // BULK IN
extern usbs_rx_endpoint usbs_at91_ep3;          // INTERRUPT OUT


static void test_usb(void)
{
USB_MSD_CONF usb_msd_conf;
int usb_token;
int stat;

	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);

	// initialize the USB mass storage driver
	//=======================================
	usb_msd_init();

	usb_msd_conf.device = "/dev/mmc";
	usb_msd_conf.endp_ctrl  = &usbs_at91_ep0;
	usb_msd_conf.endp_out   = &usbs_at91_ep1;
	usb_msd_conf.endp_in    = &usbs_at91_ep2;

	do 
	{
		stat = usb_msd_open( &usb_msd_conf, &usb_token );	
		if ( stat )
		{
			ERROR_MSG("unable to init USB Mass Storage\r\n");
			cyg_thread_delay(200);	
		}
	}
	while ( stat );		

}
#else

static void test_usb(void)
{
	printf(PREFIX_TEST"calling %s()\r\n", __FUNCTION__);
	printf(PREFIX_INFO"The USB mass storage device driver is not installed\r\n");
	printf(PREFIX_INFO"Please contact www.ronetix.com for more information\r\n");
}

#endif

//=======================================================================
//						main_task()
//						-----------
//=======================================================================
static void main_task(cyg_addrword_t data)
{

	printf("\r\nRonetix Development Kit for AT91SAM9261\r\n");

	// test the ROM filesystem
	//========================
	test_romfs();

	// test the MMC/SD driver
	//=======================
	test_mmc();

	// test the NAND Flash driver + YAFFS2
	//====================================
	test_nand_yaffs();

	// test the DataFlash driver
	//==========================
	test_dataflash();

	// test the TCP/IP stack
	//======================
	test_net();

	// test the USB + Mass Storage drivers
	//====================================
	test_usb();
	
	while(1)	
	{
		cyg_thread_delay(20);
	}

	
}


//=======================================================================
//						pio_task()
//						----------
//=======================================================================
static void pio_task(cyg_addrword_t data)
{
#define	BUTTON_SW3_PIN				AT91_GPIO_PA7			// PA7
#define	BUTTON_SW4_PIN				AT91_GPIO_PA8			// PA8
#define	LED1_PIN					AT91_GPIO_PC13			// PC13
#define	LED2_PIN					AT91_GPIO_PC15			// PC15
#define	LED3_PIN					AT91_GPIO_PC12			// PC12

cyg_uint32 sw3_val, sw3_old = 1;
cyg_uint32 sw4_val, sw4_old = 1;
int flag_toggle = false;

	// enable peripheral clock for PIOA controller
	//============================================
    HAL_WRITE_UINT32(AT91_PMC+AT91_PMC_PCER, 1 << CYGNUM_HAL_INTERRUPT_PIOA);

	// configure PA7 and PA8 as inputs
	//================================
	HAL_ARM_AT91_GPIO_CFG_DIRECTION( BUTTON_SW3_PIN, AT91_PIN_IN);	
	HAL_ARM_AT91_GPIO_CFG_DIRECTION( BUTTON_SW4_PIN, AT91_PIN_IN);	

	HAL_ARM_AT91_GPIO_CFG_DIRECTION( LED1_PIN, AT91_PIN_OUT);	
	HAL_ARM_AT91_GPIO_CFG_DIRECTION( LED2_PIN, AT91_PIN_OUT);	
	HAL_ARM_AT91_GPIO_CFG_DIRECTION( LED3_PIN, AT91_PIN_OUT);	
	
	HAL_ARM_AT91_GPIO_PUT( LED1_PIN, true );
	HAL_ARM_AT91_GPIO_PUT( LED2_PIN, true );
	HAL_ARM_AT91_GPIO_PUT( LED3_PIN, true );
	
	while(1)	
	{
		HAL_ARM_AT91_GPIO_GET( BUTTON_SW3_PIN, sw3_val );
		if ( sw3_val != sw3_old )
		{
			HAL_ARM_AT91_GPIO_PUT( LED2_PIN, sw3_val );
			printf("SW3 %s\r\n", sw3_val ? "released" : "pressed");
			sw3_old = sw3_val;
		}
			
		HAL_ARM_AT91_GPIO_GET( BUTTON_SW4_PIN, sw4_val );
		if ( sw4_val != sw4_old )
		{
			HAL_ARM_AT91_GPIO_PUT( LED3_PIN, sw4_val );
			printf("SW4 %s\r\n", sw4_val ? "released" : "pressed");
			sw4_old = sw4_val;
		}

		
		HAL_ARM_AT91_GPIO_PUT( LED1_PIN, flag_toggle );
		flag_toggle =  !flag_toggle;
		
		cyg_thread_delay(20);		// delay 200ms	
	}
}


static cyg_thread 			Main_task_thread_s;		
static char 				Main_task_stack[32*1024];		// space stacks
static cyg_handle_t 		Main_task_thread;

static cyg_thread 			Pio_task_thread_s;		
static char 				Pio_task_stack[32*1024];		// space stacks
static cyg_handle_t 		Pio_task_thread;

//==========================================================
//              void cyg_user_start(void)
//              -------------------------
//
// The entry point for an user application
//==========================================================
void cyg_user_start(void)
{
	// create main thread
	//===================
  	cyg_thread_create(	10, 						// priority
  						main_task, 					// task function
  						(cyg_addrword_t) 0,			// data for task function	
		    			"Main Task", 				// name of task
		    			(void *) Main_task_stack, 	// pointer to stack
		    			sizeof(Main_task_stack),	// size of stack
		    			&Main_task_thread, 
		    			&Main_task_thread_s
		    			);

	// create pio thread
	//==================
  	cyg_thread_create(	10, 						// priority
  						pio_task, 					// task function
  						(cyg_addrword_t) 0,			// data for task function	
		    			"PIO Task", 				// name of task
		    			(void *) Pio_task_stack, 	// pointer to stack
		    			sizeof(Pio_task_stack),	// size of stack
		    			&Pio_task_thread, 
		    			&Pio_task_thread_s
		    			);


	// start main thread
	//==================  	
  	cyg_thread_resume(Main_task_thread);
    
	// start PIO thread
	//==================  	
  	cyg_thread_resume(Pio_task_thread);
    
}



//=============================================================================
//=============================================================================




//==========================================================================
//								print_dir()
//								-----------
//	Parameter:
//		start_dir -
//
//	Return:
//		status
//
//	Description:
//		Print information about the files in "start_dir"
//==========================================================================
static int print_dir( char *start_dir )
{
int status=0, ii, total_size=0, nfiles=0, ndirs=0;
DIR *dirp;
char buff[256];
struct dirent *entry;
struct stat st;
struct tm * ptm;
cyg_fs_attrib_t fattr=0;

   	dirp = opendir( start_dir );

    if( dirp != NULL )
	{
		// Enum all dir entries
		//=====================
	    while ( 1  )
	    {
	    	// Read entry
	    	//===========
	        entry = readdir( dirp );

	        // Finished?
	        //==========
	        if( entry == NULL )
	            break;

			snprintf( buff, sizeof(buff), "%s/%s", start_dir, entry->d_name );

			// Get status
			//===========
			status = stat( buff, &st );
			if ( status ) continue;
			
			status = cyg_fs_get_attrib( buff, &fattr);
			if ( status == ENOERR ) 
			{
				if ( S_FATFS_ISSYSTEM(fattr) )
					continue;
	
				if ( S_FATFS_ISVOLUME(fattr) )
				{
					printf( "LABEL: %s\r\n", entry->d_name );
					continue;
				}
			}
			
			if ( S_ISDIR(st.st_mode) )
			{
				ndirs++;
				ptm = gmtime ( &st.st_mtime );

				// DIRs listed in upper case
				//==========================
				for ( ii=0; ii < (int)strlen(entry->d_name); ii++ )
					entry->d_name[ii] = toupper( entry->d_name[ii] );

				printf( "%02d/%02d/%04d %02d:%02d:%02d    <DIR>              %s\r\n", ptm->tm_mday, ptm->tm_mon+1, 1900+ptm->tm_year, ptm->tm_hour, ptm->tm_min, ptm->tm_sec, entry->d_name );
			}
			else
			{
				nfiles++;
				ptm = gmtime ( &st.st_mtime );

				// FILEs listed in lower case
				//===========================
				for ( ii=0; ii < (int)strlen(entry->d_name); ii++ )
					entry->d_name[ii] = tolower( entry->d_name[ii] );

				
				total_size += st.st_size;
				printf( "%02d/%02d/%04d %02d:%02d:%02d           %11ld %s\r\n", ptm->tm_mday, ptm->tm_mon+1, 1900+ptm->tm_year, ptm->tm_hour, ptm->tm_min, ptm->tm_sec, st.st_size, entry->d_name );
			}

	    }
	    
	    status = closedir( dirp );
	}

	if ( total_size < 1024 )
		printf( "           %d File(s)   %d bytes\r\n", nfiles, total_size );
	else	
		printf( "           %d File(s)   %d KB\r\n", nfiles, total_size/1024 );

	if ( ndirs > 0 )
	{	
		printf( "           %d Dir(s)\r\n", ndirs );
	}
	
	return status;
}


